import unittest
from amuse.support.interface import InCodeComponentImplementation


from amuse.test.amusetest import TestWithMPI
from amuse.test import compile_tools
import os
import time
import shlex

from amuse.units import nbody_system
from amuse.units import units
from amuse import datamodel
from amuse.rfi import channel
from amuse.rfi.core import *

codestring = """

function echo_int(int_in, int_out)
    implicit none
    integer :: int_in, int_out
    integer :: echo_int
    
    int_out = int_in
    
    echo_int = 0
end function

function echo_double(double_in, double_out)
    implicit none
    DOUBLE PRECISION :: double_in, double_out
    integer :: echo_double
    
    double_out = double_in
    
    echo_double = 0
end function

function echo_float(float_in, float_out)
    implicit none
    REAL(kind=4) :: float_in, float_out
    integer :: echo_float
    
    float_out = float_in
    
    echo_float = 0
end function

function echo_string(string_in, string_out)
    implicit none
    character(len=*) :: string_in, string_out
    integer :: echo_string
    
    string_out = string_in
    
    echo_string = 0
end function

function echo_strings(string_inout1, string_inout2)
    implicit none
    character(len=*) :: string_inout1, string_inout2
    integer :: echo_strings
    
    string_inout1(1:1) = 'A'
    string_inout2(1:1) = 'B'
    
    
    echo_strings = 0
end function


function return_string(string_in)
    implicit none
    character(len=*) :: string_in, return_string
    
    return_string = string_in
end function


function hello_string(string_out)
    implicit none
    character(len=*) :: string_out
    integer :: hello_string
    
    string_out = 'hello'
    
    hello_string = 0
end function

function print_string(string_in)
    implicit none
    character(len=*) :: string_in
    integer :: print_string
    
    write (*,*) string_in
    
    print_string = 0
end function

function print_error_string(string_in)
    implicit none
    character(len=*) :: string_in
    integer :: print_error_string
    
    write (0,*) string_in
    
    print_error_string = 0
end function

function echo_string_fixed_len(string_in, string_out)
    implicit none
    character(len=30) :: string_in, string_out
    integer :: echo_string_fixed_len
    
    string_out = string_in
    
    echo_string_fixed_len = 0
end function

function echo_array_with_result(int_in, int_out, N)
    implicit none
    integer, intent(in) :: N
    integer :: int_in(N), int_out(N)
    integer :: echo_array_with_result,  i
    
    do i = 1, N
     int_out(i) = int_in(i)
    end do
    
    echo_array_with_result = -1
end function



function echo_inout_array_with_result(inout, N) 
    implicit none
    integer, intent(in) :: N
    integer :: inout(N)
    integer :: echo_inout_array_with_result,  i
    
    do i = 1, N
     inout(i) = inout(i) + 10
    end do
    
    echo_inout_array_with_result = 11;
end function


function echo_logical(input, output)
    implicit none
    logical :: input, output
    integer :: echo_logical
    
    output = input
    print *, "INPUT=", input
    
    echo_logical = 0
end function

function echo_logical2(input, output, n)
    implicit none
    logical :: input(n), output(n)
    integer :: echo_logical2, n,i
    
    output(1:n)=.FALSE.
    do i=1,n
      if(input(i)) then
        output(i) = .TRUE.
      endif
    enddo
    
    echo_logical2 = 0
end function


function get_element_status(ind,x) result(ret)
  integer :: ind,ret
  character(len=*) :: x
  x="dry"
  ret=0
end function
"""


class ForTestingInterface(CodeInterface):

    def __init__(self, exefile, **options):
        CodeInterface.__init__(self, exefile, **options)

    # use_modules=['code']

    @legacy_function
    def echo_int():
        function = LegacyFunctionSpecification()
        function.addParameter('int_in', dtype='int32', direction=function.IN, unit=units.m)
        function.addParameter('int_out', dtype='int32', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_double():
        function = LegacyFunctionSpecification()
        function.addParameter('double_in', dtype='float64', direction=function.IN)
        function.addParameter('double_out', dtype='float64', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_float():
        function = LegacyFunctionSpecification()
        function.addParameter('float_in', dtype='float32', direction=function.IN)
        function.addParameter('float_out', dtype='float32', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_string():
        function = LegacyFunctionSpecification()
        function.addParameter('string_in', dtype='string', direction=function.IN)
        function.addParameter('string_out', dtype='string', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_strings():
        function = LegacyFunctionSpecification()
        function.addParameter('string_inout1', dtype='string', direction=function.INOUT)
        function.addParameter('string_inout2', dtype='string', direction=function.INOUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def return_string():
        function = LegacyFunctionSpecification()
        function.addParameter('string_in', dtype='string', direction=function.IN)
        function.result_type = 'string'
        function.can_handle_array = True
        return function

    @legacy_function
    def hello_string():
        function = LegacyFunctionSpecification()
        function.addParameter('string_out', dtype='string', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_string_fixed_len():
        function = LegacyFunctionSpecification()
        function.addParameter('string_in', dtype='string', direction=function.IN)
        function.addParameter('string_out', dtype='string', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_array_with_result():
        function = LegacyFunctionSpecification()
        function.addParameter('int_in', dtype='int32', direction=function.IN)
        function.addParameter('int_out', dtype='int32', direction=function.OUT)
        function.addParameter('len', dtype='int32', direction=function.LENGTH)
        function.result_type = 'int32'
        function.must_handle_array = True
        return function

    @legacy_function
    def echo_inout_array_with_result():
        function = LegacyFunctionSpecification()
        function.addParameter('in_out', dtype='int32', direction=function.INOUT)
        function.addParameter('len', dtype='int32', direction=function.LENGTH)
        function.result_type = 'int32'
        function.must_handle_array = True
        return function

    @legacy_function
    def echo_logical():
        function = LegacyFunctionSpecification()
        function.addParameter('input', dtype='bool', direction=function.IN)
        function.addParameter('output', dtype='bool', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def echo_logical2():
        function = LegacyFunctionSpecification()
        function.addParameter('input', dtype='bool', direction=function.IN)
        function.addParameter('output', dtype='bool', direction=function.OUT)
        function.addParameter('n', dtype='int32', direction=function.LENGTH)
        function.result_type = 'int32'
        function.must_handle_array = True
        return function

    @legacy_function
    def print_string():
        function = LegacyFunctionSpecification()
        function.addParameter('string_in', dtype='string', direction=function.IN)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def print_error_string():
        function = LegacyFunctionSpecification()
        function.addParameter('string_in', dtype='string', direction=function.IN)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function

    @legacy_function
    def get_element_status():
        function = LegacyFunctionSpecification()
        function.addParameter('index', dtype='i', direction=function.IN)
        function.addParameter('status', dtype='string', direction=function.OUT)
        function.result_type = 'int32'
        function.can_handle_array = True
        return function


class ForTesting(InCodeComponentImplementation):

    def __init__(self, exefile, **options):
        InCodeComponentImplementation.__init__(self, ForTestingInterface(exefile, **options), **options)

    """
    def define_methods(self, object):
        object.add_method(
            'echo_int',
            (units.m,)
            ,
            (
                units.m,
                object.ERROR_CODE,
            )
        )    
    """


class TestInterface(TestWithMPI):

    @classmethod
    def skip_if_fortran_does_not_support_mpi(cls):
        try:
            from amuse import config
            is_configured = hasattr(config, 'compilers') and hasattr(config.compilers, 'fc_iso_c_bindings')
        except ImportError:
            is_configured = False

        if is_configured and config.compilers.fc_iso_c_bindings:
            return
        else:
            cls.skip("cannot run test as fortran does not support iso c bindings")

    @classmethod
    def setup_class(cls):
        print("building")
        cls.check_can_compile_modules()
        cls.exefile = compile_tools.build_fortran_worker(codestring, cls.get_path_to_results(), ForTestingInterface)
        print("done")

    def test1(self):
        instance = ForTestingInterface(self.exefile)
        int_out, error = instance.echo_int(10)
        del instance
        self.assertEqual(int_out, 10)
        self.assertEqual(error, 0)

    def test2(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_double(4.0)
        del instance
        self.assertEqual(out, 4.0)
        self.assertEqual(error, 0)

    def test3(self):
        instance = ForTestingInterface(self.exefile)
        input = [1, 2, 3, 4]
        output, errors = instance.echo_int(input)
        del instance
        self.assertEqual(len(errors), 4)
        for actual, expected in zip(output, input):
            self.assertEqual(actual, expected)

    def test4(self):
        instance = ForTestingInterface(self.exefile)
        input = [1.0, 2.1, 3.3, 4.2]
        output, errors = instance.echo_double(input)
        del instance
        self.assertEqual(len(errors), 4)
        for actual, expected in zip(output, input):
            self.assertEqual(actual, expected)

    def test5(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_float(4.0)
        del instance
        self.assertEqual(out, 4.0)
        self.assertEqual(error, 0)

    def test6(self):

        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_string("abc")
        del instance
        self.assertEqual(error, 0)
        self.assertEqual(out, "abc")

    def test7(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_string(["abc", "def"])
        del instance

        self.assertEqual(error[0], 0)
        self.assertEqual(error[1], 0)
        self.assertEqual(out[0], "abc")
        self.assertEqual(out[1], "def")

    def test8(self):
        instance = ForTestingInterface(self.exefile)
        out1, out2, error = instance.echo_strings("abc", "def")
        del instance

        self.assertEqual(error, 0)
        self.assertEqual(out1, "Abc")
        self.assertEqual(out2, "Bef")

    def test9(self):
        instance = ForTestingInterface(self.exefile)
        str1_out, str2_out, error = instance.echo_strings(["abc", "def"], ["ghi", "jkl"])
        del instance

        self.assertEqual(error[0], 0)
        self.assertEqual(error[1], 0)
        self.assertEqual(str1_out[0], "Abc")
        self.assertEqual(str1_out[1], "Aef")
        self.assertEqual(str2_out[0], "Bhi")
        self.assertEqual(str2_out[1], "Bkl")

    def test10(self):
        instance = ForTestingInterface(self.exefile)
        out = instance.return_string("abc")
        del instance

        self.assertEqual(out, "abc")

    def test11(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.hello_string()
        del instance

        self.assertEqual(out, "hello")

    def test12(self):

        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_string_fixed_len("abc")
        del instance
        self.assertEqual(error, 0)
        self.assertEqual(out, "abc")

    def test13(self):
        instance = ForTestingInterface(self.exefile, debugger="none")
        (output_ints, error) = instance.echo_array_with_result([4, 5, 6])
        instance.stop()
        print(output_ints, error)
        self.assertEqual(output_ints[0], 4)
        self.assertEqual(output_ints[1], 5)
        self.assertEqual(output_ints[2], 6)

        self.assertEqual(error[0], -1)
        self.assertEqual(error[1], -1)
        self.assertEqual(error[2], -1)

    def test14(self):

        for x in range(4):
            instance = ForTestingInterface(self.exefile)
            int_out, error = instance.echo_int(10)
            instance.stop()
            self.assertEqual(int_out, 10)
            self.assertEqual(error, 0)

    def test15(self):
        instance = ForTestingInterface(self.exefile)
        (output_ints, error) = instance.echo_inout_array_with_result([4, 5, 6])
        instance.stop()
        self.assertEqual(output_ints[0], 14)
        self.assertEqual(output_ints[1], 15)
        self.assertEqual(output_ints[2], 16)

        self.assertEqual(error[0], 11)
        self.assertEqual(error[1], 11)
        self.assertEqual(error[2], 11)

    def test16(self):
        instance = ForTestingInterface(self.exefile)
        (output1, error1) = instance.echo_logical(True)
        (output2, error2) = instance.echo_logical(False)
        instance.stop()
        self.assertEqual(error1, 0)
        self.assertEqual(error2, 0)
        self.assertTrue(output1)
        self.assertFalse(output2)

    def test16b(self):
        instance = ForTesting(self.exefile)
        output = instance.echo_logical([True, True, False, True, False]*256)
        self.assertEqual(output, [True, True, False, True, False]*256)

    def test16c(self):
        instance = ForTesting(self.exefile, redirection="none")
        output = instance.echo_logical2([True, True, False, True, False]*1024)
        self.assertEqual(output, [True, True, False, True, False]*1024)

    def xtest20(self):
        #
        # TURNED OFF support for redirection,
        # by default output is redirected to /dev/null
        # if you need file, use the support from your mpi implementation
        #
        if os.path.exists("pout.000"):
            os.remove("pout.000")
        if os.path.exists("perr.000"):
            os.remove("perr.000")

        x = ForTestingInterface(self.exefile, redirect_stderr_file='perr', redirect_stdout_file='pout', redirection="file")
        x.print_string("abc")
        x.print_error_string("exex")
        x.stop()

        time.sleep(0.2)

        self.assertTrue(os.path.exists("pout.000"))
        with open("pout.000", "r") as f:
            content = f.read()
        self.assertEqual(content.strip(), "abc")

        self.assertTrue(os.path.exists("perr.000"))
        with open("perr.000", "r") as f:
            content = f.read()
        self.assertEqual(content.strip(), "exex")

        x = ForTestingInterface(self.exefile, redirect_stderr_file='perr', redirect_stdout_file='pout', redirection="file")
        x.print_string("def")
        x.print_error_string("exex")
        x.stop()

        time.sleep(0.2)

        self.assertTrue(os.path.exists("pout.000"))
        with open("pout.000", "r") as f:
            content = f.read()
        self.assertEqual(content.strip(), "abc\n def")

        self.assertTrue(os.path.exists("perr.000"))
        with open("perr.000", "r") as f:
            content = f.read()
        self.assertEqual(content.strip(), "exex\n exex")

    def test17(self):
        self.check_for_mpi()
        instance = ForTestingInterface(self.exefile)
        (output1, error1) = instance.internal__get_message_polling_interval()
        error2 = instance.internal__set_message_polling_interval(1234)
        (output2, error3) = instance.internal__get_message_polling_interval()
        instance.stop()
        self.assertEqual(error1, 0)
        self.assertEqual(output1, 0)
        self.assertEqual(error2, 0)
        self.assertEqual(error3, 0)
        self.assertEqual(output2, 1234)

    def test18(self):
        self.check_for_mpi()
        self.skip_if_fortran_does_not_support_mpi()

        instance = ForTestingInterface(self.exefile)
        t0 = time.time()
        (output1, error1) = instance.internal__get_message_polling_interval()
        t1 = time.time()
        error2 = instance.internal__set_message_polling_interval(500 * 1000)
        t2 = time.time()
        (output2, error3) = instance.internal__get_message_polling_interval()
        time.sleep(0.1)
        (output2, error3) = instance.internal__get_message_polling_interval()
        t3 = time.time()
        instance.stop()
        self.assertEqual(error1, 0)
        self.assertEqual(output1, 0)
        self.assertEqual(error2, 0)
        self.assertEqual(error3, 0)
        self.assertEqual(output2, 500 * 1000)
        # ~ print t1 - t0, t3 - t2
        # ~ self.assertTrue((t3 - t2) > 0.25)

    def test19(self):
        self.check_for_mpi()
        instance1 = ForTestingInterface(self.exefile)
        instance2 = ForTestingInterface(self.exefile)
        portname, error = instance1.internal__open_port()
        self.assertTrue(len(portname) > 0)
        request1 = instance1.internal__accept_on_port.asynchronous(portname)
        request2 = instance2.internal__connect_to_port.asynchronous(portname)
        request1.wait()
        request2.wait()
        port_id1, error1 = request1.result()
        port_id2, error2 = request2.result()
        self.assertTrue(port_id1 >= 0)
        self.assertTrue(port_id2 >= 0)
        self.assertEqual(error1, 0)
        self.assertEqual(error2, 0)

    @unittest.skip
    def test31(self):
        import time
        instance = ForTestingInterface(self.exefile)
        N = 5000
        t1 = time.time()
        for i in range(N):
            res, err = instance.echo_int([i])
        t2 = time.time()
        print("1 time:", t2-t1, (t2-t1)/N)
        instance.stop()

        instance = ForTesting(self.exefile)
        N = 5000
        t1 = time.time()
        for i in range(N):
            res = instance.echo_int([i] | units.m)
        t2 = time.time()
        print("2 time:", t2-t1, (t2-t1)/N)
        instance.stop()

    def test32(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.get_element_status(numpy.arange(10))
        del instance

        self.assertEqual(out, ["dry"]*10)

    def test33(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.get_element_status(numpy.arange(100))
        del instance

        self.assertEqual(out, ["dry"]*100)

    def test34(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_string(["abc"]*14)
        del instance

        self.assertEqual(out, ["abc"]*14)
        self.assertEqual(error, [0]*14)

    def test35(self):
        instance = ForTestingInterface(self.exefile)
        out, error = instance.echo_string(["abc", "def"]*100000)
        del instance

        self.assertEqual(error[0], 0)
        self.assertEqual(error[1], 0)
        self.assertEqual(out[-2], "abc")
        self.assertEqual(out[-1], "def")

    def test36(self):
        instance = ForTestingInterface(self.exefile)
        N = 255
        out, error = instance.echo_string("a"*N)
        del instance

        self.assertEqual(error, 0)
        self.assertEqual(out, "a"*N)



import numpy
from amuse.test.amusetest import TestCase


from amuse.ext.solarsystem import new_solar_system, new_solar_system_for_mercury
from amuse.units import units
from amuse.units import constants


class TestSolarSystem(TestCase):

    def test1(self):
        print("Test 1: testing new_solar_system_for_mercury")
        sun, orbiters = new_solar_system_for_mercury()

        expected_attributes = set(["name", "mass", "radius", "j2", "j4", "j6", "lx", "ly", "lz"])
        self.assertEqual(set(sun.get_attribute_names_defined_in_store()), expected_attributes)

        expected_attributes = set(["name", "mass", "radius", "density", "x", "y", "z", "vx", "vy", "vz", "Lx", "Ly", "Lz", "celimit"])
        self.assertEqual(set(orbiters.get_attribute_names_defined_in_store()), expected_attributes)

    def test2(self):
        print("Test 2: testing new_solar_system")
        particles = new_solar_system()
        print(particles)

        expected_attributes = set(["name", "mass", "radius", "x", "y", "z", "vx", "vy", "vz"])
        self.assertEqual(set(particles.get_attribute_names_defined_in_store()), expected_attributes)

        self.assertAlmostEqual(particles.center_of_mass(), [0, 0, 0] | units.m, in_units=units.AU)
        self.assertAlmostEqual(particles.center_of_mass_velocity(), [0, 0, 0] | units.m/units.s, in_units=units.AUd)
        # Particles are in center-of-mass(-velocity) coordinates, move them to heliocentric coordinates:
        self.assertTrue(particles[0].name == "SUN")
        particles.position -= particles[0].position
        particles.velocity -= particles[0].velocity

        # Data from Carroll & Ostlie, An introduction to modern astrophysics, 1996
        eccentricity = numpy.asarray([0.2056, 0.0068, 0.0167, 0.0934, 0.0483, 0.0560, 0.0461, 0.0097, 0.2482])
        semimajor_axis = [0.3871, 0.7233, 1.0000, 1.5237, 5.2028, 9.5388, 19.1914, 30.0611, 39.5294] | units.AU

        self.assertAlmostRelativeEqual(particles[2:-1].position.lengths_squared(), semimajor_axis[1:-1]**2, 1)

        # Somewhat more complicated test for more eccentric orbiters Mercury and Pluto:
        expected = (constants.G * (particles[1:].mass + particles[0].mass) * semimajor_axis * (1 - eccentricity**2)).sqrt()
        self.assertAlmostRelativeEqual(particles[1:].position.cross(particles[1:].velocity).lengths(), expected, 2)



import numpy
import math
import os

from amuse.test import amusetest
from amuse.io import store
from amuse.units import units
from amuse.units import nbody_system
from amuse.datamodel import *
from amuse.datamodel import incode_storage


class TestAttributeError(amusetest.TestCase):

    def new_code_particles(self):
        class Code(object):
            def __init__(self):
                # mass
                self.data = []
                self.get_mass_called = False
                self.set_mass_called = False
                self.number_of_particles = 0

            def get_number_of_particles(self):
                return self.number_of_particles

            def get_mass(self, index):
                self.get_mass_called = True
                data_to_return = [self.data[i] for i in index]
                return units.kg(data_to_return)

            def set_mass(self, index, mass):
                self.set_mass_called = True
                pass

            def new_particle(self, mass):
                mass = mass.value_in(units.kg)
                self.data = mass
                self.number_of_particles = len(self.data)
                return [i for i in range(len(mass))]

        code = Code()
        storage = incode_storage.InCodeAttributeStorage(
            code,
            incode_storage.NewParticleMethod(code.new_particle, ("mass",)),
            None,
            code.get_number_of_particles,
            [],
            [
                incode_storage.ParticleGetAttributesMethod(code.get_mass, ("mass",)),
            ],
            name_of_the_index="index"
        )

        return Particles(storage=storage)

    def test1(self):
        print("Test1: Should get error when accessing non-existent attributes (InMemoryAttributeStorage).")
        particles = Particles(4)
        particle = Particle()
        subset = particles[:2]
        superset = ParticlesSuperset([particles, particle.as_set()])

        instances = [particles, particle, subset, superset]
        classes = [Particles, Particle, ParticlesSubset, ParticlesSuperset]
        lengths = [4, 1, 2, 5]
        for i, x in enumerate(instances):
            self.assertTrue(isinstance(x, classes[i]))
            self.assertEqual(len(x.as_set()), lengths[i])
            self.assertRaises(AttributeError, lambda: x.bogus, expected_message="You tried to access attribute 'bogus' but this attribute is not defined for this set.")

    def test2(self):
        print("Test2: Should get error when accessing non-existent attributes (in Legacy code storage).")
        particles = Particles(4)
        particle = Particle()
        subset = particles[:2]
        superset = ParticlesSuperset([particles, particle.as_set()])
        superset.mass = 1.0 | units.MSun
        superset.radius = 1.0 | units.RSun
        for i, x in enumerate(superset):
            x.mass = 2.0 | units.kg
        instances = [particles, particle.as_set(), subset, superset]
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 1.0 | units.AU)
        for i, x in enumerate(instances):
            code_particles = self.new_code_particles()
            code_particles.add_particles(x)
            self.assertRaises(AttributeError, lambda: code_particles.bogus, expected_message="You tried to access attribute 'bogus' but this attribute is not defined for this set.")

    def test3(self):
        print("Test3: Should get error when accessing non-existent attributes (HDF5 storage).")
        particles = Particles(4)
        particle = Particle()
        subset = particles[:2]
        superset = ParticlesSuperset([particles, particle.as_set()])
        superset.mass = 1.0 | units.MSun
        test_results_path = self.get_path_to_results()
        output_file = os.path.join(test_results_path, "attr_test.hdf5")

        instances = [particles, particle, subset]
        classes = [Particles, Particle, ParticlesSubset]
        lengths = [4, 1, 2]
        for i, x in enumerate(instances):
            self.assertTrue(isinstance(x, classes[i]))
            self.assertEqual(len(x.as_set()), lengths[i])
            if os.path.exists(output_file):
                os.remove(output_file)
            HDFstorage = store.StoreHDF(output_file)
            if isinstance(x, Particle):
                x = x.as_set()
            x.model_time = 2.0 | units.s
            HDFstorage.store(x)
            loaded_particles = HDFstorage.load()
            self.assertRaises(AttributeError, lambda: loaded_particles.bogus, expected_message="You tried to access attribute 'bogus' but this attribute is not defined for this set.")
            HDFstorage.close()
            del HDFstorage

    def bogus_func(self, x):
        x.mass = 1.0

    def xtest4(self):
        print("Test4: Should get error when setting attributes with non-quantities (InMemoryAttributeStorage).")
        particles = Particles(4)
        particle = Particle()
        subset = particles[:2]
        superset = ParticlesSuperset([particles, particle.as_set()])

        instances = [particles, particle, subset, superset]
        classes = [Particles, Particle, ParticlesSubset, ParticlesSuperset]
        lengths = [4, 1, 2, 5]
        for i, x in enumerate(instances):
            self.assertTrue(isinstance(x, classes[i]))
            self.assertEqual(len(x.as_set()), lengths[i])
            self.assertRaises(AttributeError, self.bogus_func, x, expected_message="Can only assign quantities or other particles to an attribute.")

    def xtest5(self):
        print("Test5: Should get error when setting attributes with non-quantities (in code storage).")
        particles = Particles(4)
        particle = Particle()
        subset = particles[:2]
        superset = ParticlesSuperset([particles, particle.as_set()])
        superset.mass = 1.0 | units.MSun
        for i, x in enumerate(superset):
            x.mass = 2.0 | units.kg
        instances = [particles, particle.as_set(), subset, superset]
        classes = [Particles, Particle, ParticlesSubset, ParticlesSuperset]
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 1.0 | units.AU)
        for i, x in enumerate(instances):
            code_particles = self.new_code_particles()
            code_particles.add_particles(x)
            # self.assertRaises(AttributeError, self.bogus_func, code_particles, expected_message =
            #    "Can only assign quantities or other particles to an attribute.")

    def xtest6(self):
        print("Test6: Should get error when setting attributes with non-quantities (HDF5 storage).")
        particles = Particles(4)
        particle = Particle()
        subset = particles[:2]
        superset = ParticlesSuperset([particles, particle.as_set()])
        superset.mass = 1.0 | units.MSun
        test_results_path = self.get_path_to_results()
        output_file = os.path.join(test_results_path, "attr_test.hdf5")
        instances = [particles, particle, subset]
        classes = [Particles, Particle, ParticlesSubset]
        lengths = [4, 1, 2]
        for i, x in enumerate(instances):
            self.assertTrue(isinstance(x, classes[i]))
            self.assertEqual(len(x.as_set()), lengths[i])
            if os.path.exists(output_file):
                os.remove(output_file)
            HDFstorage = store.StoreHDF(output_file)
            if isinstance(x, Particle):
                x = x.as_set()
            x.model_time = 2.0 | units.s
            HDFstorage.store(x)
            loaded_particles = HDFstorage.load()
            self.assertRaises(AttributeError, self.bogus_func, loaded_particles, expected_message="Can only assign quantities or other particles to an attribute.")
            HDFstorage.close()
            del HDFstorage

    def test7(self):
        print("Test7: testing whether nonexistent particle attribute access raises correct error")
        p = Particles(2)
        q = p[0]
        p.remove_particle(p[0])
        self.assertRaises(exceptions.KeysNotInStorageException, lambda: q.position)



import os
import sys

from amuse.test import amusetest
from amuse.community.bhtree.interface import BHTreeInterface, BHTree
from amuse.community.hermite.interface import HermiteInterface, Hermite
from amuse.community.phigrape.interface import PhiGRAPEInterface, PhiGRAPE
from amuse.units import nbody_system
from amuse.units import units
from amuse import datamodel


class TestParameterDoc(amusetest.TestCase):

    def test_bhtree(self):

        convert_nbody = nbody_system.nbody_to_si(1.0 | units.kg, 1.0 | units.km)

        bhtree = BHTree(convert_nbody)

        bhtree.parameters.epsilon_squared = 10 | units.km**2
        bhtree.parameters.timestep = 1.0 | units.s
        bhtree.parameters.opening_angle = 0.1

        docstring = bhtree.parameters.__doc__

        self.assertTrue("smoothing parameter for gravity calculations (default value:125000.0 m**2)" in docstring)

        parameter_str_method_output = str(bhtree.parameters)

        self.assertTrue("epsilon_squared: 10000000.0 m**2" in parameter_str_method_output)
        self.assertTrue("timestep: 1.0 s" in parameter_str_method_output)
        self.assertTrue("opening_angle: 0.1" in parameter_str_method_output)

    def test_hermite(self):

        convert_nbody = nbody_system.nbody_to_si(1.0 | units.kg, 1.0 | units.km)

        hermite = Hermite(convert_nbody)

        hermite.parameters.epsilon_squared = 10 | units.km**2

        docstring = hermite.parameters.__doc__
        self.assertTrue("smoothing parameter for gravity calculations (default value:0.0 m**2)" in docstring)

        parameter_str_method_output = str(hermite.parameters)
        self.assertTrue("epsilon_squared: 10000000.0 m**2" in parameter_str_method_output)

    def test_phigrape(self):

        convert_nbody = nbody_system.nbody_to_si(1.0 | units.kg, 1.0 | units.km)

        phigrape = PhiGRAPE(convert_nbody)
        phigrape.parameters.epsilon_squared = 10 | units.km**2

        docstring = phigrape.parameters.__doc__
        print(docstring)
        self.assertTrue("smoothing parameter for gravity calculations (default value:0.0 m**2)" in docstring)
        self.assertTrue("timestep parameter (default value:0.02" in docstring)
        self.assertTrue("parameter to determine the initial timestep (default value:0.01" in docstring)

        parameter_str_method_output = str(phigrape.parameters)
        self.assertTrue("epsilon_squared: 10000000.0 m**2" in parameter_str_method_output)
        self.assertTrue("timestep_parameter: 0.0" in parameter_str_method_output)
        self.assertTrue("initial_timestep_parameter: 0.0" in parameter_str_method_output)



from amuse.support import options
from amuse.test import amusetest
import io
import textwrap
import os


class OptionsTestsClass(options.OptionalAttributes):
    option_sections = ('amuse',)

    def string_option(self):
        return "default string"

    def boolean_option(self):
        return False

    def int_option(self):
        return 2

    def float_option(self):
        return 2.5


class OptionsTests(amusetest.TestCase):
    ini_contents = textwrap.dedent("""
    [amuse]
    string_option=a string
    boolean_option=1
    int_option=1
    float_option=1.5
    [bhtree]
    mode=gpu
    int_option=3
    """)

    def test1(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))
        instance = OptionsTestsClass()

        option = options.option(OptionsTestsClass.string_option, global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), "a string")

    def test2(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))
        instance = OptionsTestsClass()

        option = options.option(OptionsTestsClass.int_option, type="int", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), 1)

        option = options.option(OptionsTestsClass.boolean_option, type="boolean", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), True)

        option = options.option(OptionsTestsClass.float_option, type="float", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), 1.5)

    def test3(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))
        instance = OptionsTestsClass()
        instance.option_sections = ('unknown')

        option = options.option(OptionsTestsClass.string_option, global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), "default string")

        option = options.option(OptionsTestsClass.int_option, type="int", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), 2)

        option = options.option(OptionsTestsClass.boolean_option, type="boolean", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), False)

        option = options.option(OptionsTestsClass.float_option, type="float", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), 2.5)

    def test4(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))
        instance = OptionsTestsClass()

        option = options.option(OptionsTestsClass.int_option, type="int", sections=("bhtree",), global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), 1)

        instance.option_sections = ('unknown')
        self.assertEqual(option.__get__(instance, type(instance)), 3)

    def test5(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))
        instance = OptionsTestsClass()
        instance.option_sections = ('unknown')

        option = options.option(type="int", global_options=global_options)
        option = option(OptionsTestsClass.int_option)

        self.assertEqual(option.__get__(instance, type(instance)), 2)

    def test6(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))

        class DecoratedMethods(options.OptionalAttributes):
            option_sections = ('unknown')

            @options.option(type="int", global_options=global_options)
            def int_option(self):
                return self.i

        instance = DecoratedMethods()
        instance.i = 10
        self.assertEqual(instance.int_option, 10)

    def test7(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))

        class DecoratedMethods(options.OptionalAttributes):
            option_sections = ('unknown')
            i = 8

            def __init__(self, **optional_arguments):
                options.OptionalAttributes.__init__(self, **optional_arguments)

            @options.option(type="int", global_options=global_options)
            def int_option(self):
                return self.i

        instance = DecoratedMethods()
        instance.i = 10
        instance.int_option = 12
        self.assertEqual(instance.int_option, 12)

        instance = DecoratedMethods(int_option=14)
        self.assertEqual(instance.int_option, 14)

    def test8(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))

        class A(options.OptionalAttributes):
            option_sections = ('amuse')

            def __init__(self, **optional_arguments):
                options.OptionalAttributes.__init__(self, **optional_arguments)

            @options.option(type="int", global_options=global_options)
            def int_option_a(self):
                return self.i

        class B(options.OptionalAttributes):
            option_sections = ('amuse')

            def __init__(self, **optional_arguments):
                options.OptionalAttributes.__init__(self, **optional_arguments)

            @options.option(type="int", global_options=global_options)
            def int_option_b(self):
                return 10

        class C(A, B):

            def __init__(self, **optional_arguments):
                A.__init__(self, **optional_arguments)
                B.__init__(self, **optional_arguments)

        instance = C(int_option_a=14)
        self.assertEqual(instance.int_option_a, 14)
        self.assertEqual(instance.int_option_b, 10)

    def test9(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))

        class A(options.OptionalAttributes):
            option_sections = ('amuse')

            def __init__(self, **optional_arguments):
                options.OptionalAttributes.__init__(self, **optional_arguments)

            @options.option(type="int", global_options=global_options)
            def int_option_a(self):
                return self.i

        class B(options.OptionalAttributes):
            option_sections = ('amuse')

            def __init__(self, **optional_arguments):
                options.OptionalAttributes.__init__(self, **optional_arguments)

            @options.option(type="int", global_options=global_options)
            def int_option_b(self):
                return 10

        class C(A, B):

            def __init__(self, **optional_arguments):
                A.__init__(self, **optional_arguments)
                B.__init__(self, **optional_arguments)

        instance = C(int_option_a=14)
        self.assertEqual(instance.int_option_a, 14)
        self.assertEqual(instance.int_option_b, 10)

    def test10(self):
        global_options = options.GlobalOptions()
        self.assertEqual(global_options.rcfilename, 'amuserc')
        global_options.rcfilename = 'test_for_amuserc'
        with open(global_options.rcfilename, "w") as f:
            f.write(self.ini_contents)

        self.assertTrue(global_options.rcfilepath.endswith('test_for_amuserc'))
        global_options.load()

        instance = OptionsTestsClass()
        option = options.option(OptionsTestsClass.int_option, type="int", global_options=global_options)
        self.assertEqual(option.__get__(instance, type(instance)), 1)

        os.remove(global_options.rcfilepath)

    def test11(self):
        global_options = options.GlobalOptions()
        global_options.config.read_file(io.StringIO(self.ini_contents))

        class DecoratedMethods(options.OptionalAttributes):
            option_sections = ('amuse',)

            def __init__(self, **optional_arguments):
                options.OptionalAttributes.__init__(self, **optional_arguments)

            @options.option(choices=("1", "2"), global_options=global_options)
            def int_option(self):
                return 2

        instance = DecoratedMethods()
        self.assertEqual(instance.int_option, "1")
        all_options = list(instance.iter_options())
        self.assertEqual(len(all_options), 1)
        self.assertEqual(all_options[0].name, "int_option")

    def test12(self):
        global_options = options.GlobalOptions()
        global_options.read_from_ini_string(self.ini_contents)
        print(global_options.to_ini_string())
        ini_string = global_options.to_ini_string()

        self.assertTrue(ini_string.find("= a string") > 0)



from amuse.community import *
from amuse.test.amusetest import TestWithMPI

from amuse.community.nbody6xx.interface import Nbody6xxInterface
from amuse.community.nbody6xx.interface import Nbody6xx

import math


class Nbody6xxInterfaceTests(TestWithMPI):

    def test1(self):
        instance = Nbody6xxInterface()
        instance.initialize_code()
        res1 = instance.new_particle(mass=11.0, radius=2.0, x=0.0, y=0.0, z=0.0, vx=0.0, vy=0.0, vz=0.0)
        res2 = instance.new_particle(mass=21.0, radius=5.0, x=10.0, y=0.0, z=0.0, vx=10.0, vy=0.0, vz=0.0)

        self.assertEqual(1, res1['index_of_the_particle'])
        self.assertEqual(2, res2['index_of_the_particle'])

        retrieved_state1 = instance.get_state(1)
        retrieved_state2 = instance.get_state(2)

        self.assertEqual(11.0,  retrieved_state1['mass'])
        self.assertEqual(21.0,  retrieved_state2['mass'])
        self.assertEqual(0.0,  retrieved_state1['x'])
        self.assertEqual(10.0,  retrieved_state2['x'])
        # self.assertEquals(2.0,  retrieved_state1['radius'])
        # self.assertEquals(5.0,  retrieved_state2['radius'])

        instance.cleanup_code()
        instance.stop()

    def test2(self):
        instance = Nbody6xxInterface()
        instance.initialize_code()

        instance.new_particle([10, 20], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [1, 1])
        retrieved_state = instance.get_state(1)

        self.assertEqual(10.0,  retrieved_state['mass'])
        # self.assertEquals(1, retrieved_state['radius'])

        retrieved_state = instance.get_state([1, 2])

        self.assertEqual(20.0,  retrieved_state['mass'][1])
        self.assertEqual(instance.get_number_of_particles()['number_of_particles'], 2)
        instance.cleanup_code()
        instance.stop()

    def xtest3(self):
        instance = Nbody6xxInterface(redirection="none")  # ,debugger="ddd")
        self.assertEqual(0, instance.initialize_code())
        self.assertEqual(0, instance.commit_parameters())

        # Set up an equal-mass binary on a circular orbit:
        self.assertEqual([1, 0], list(instance.new_particle(0.5,  0.5, 0, 0,  0, 0.5, 0, 0.01).values()))
        self.assertEqual([2, 0], list(instance.new_particle(0.5,  -0.5, 0, 0,  0, -0.5, 0, 0.01).values()))
        self.assertEqual([3, 0], list(instance.new_particle(100.5,  -0.5, 0, 0,  0, -0.5, 0, 0.01).values()))
        self.assertEqual(0, instance.commit_particles())

        self.assertEqual(0, instance.evolve_model(math.pi))
        for result, expected in zip(instance.get_position(1).values(), [-0.5, 0.0, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)
        for result, expected in zip(instance.get_position(2).values(), [0.5, 0.0, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)

        self.assertEqual(0, instance.evolve_model(2 * math.pi))
        for result, expected in zip(instance.get_position(1).values(), [0.5, 0.0, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)
        for result, expected in zip(instance.get_position(2).values(), [-0.5, 0.0, 0.0, 0]):
            self.assertAlmostEqual(result, expected, 3)

        self.assertEqual(0, instance.cleanup_code())
        instance.stop()



import numpy

from amuse.test import amusetest
from amuse.units import nbody_system
from amuse.units import units
from amuse.ic.mameclot import mameclot


class TestPlummer(amusetest.TestCase):
    def test1(self):
        cluster = mameclot().result

        self.assertAlmostEqual(cluster.total_mass().number, 1.0)
        self.assertEqual(len(cluster), 10000)

    def test2(self):
        clusters, c1, c2 = mameclot().result_split

        self.assertAlmostEqual((c1.total_mass()+c2.total_mass()).number, 1.0)
        self.assertEqual(len(c1), 10000)
        self.assertEqual(len(c2), 0)

    def test3(self):
        clusters, c1, c2 = mameclot(mass_ratio=0.25).result_split

        self.assertAlmostEqual((c1.total_mass()+c2.total_mass()).number, 1.0)
        self.assertEqual(len(c1), 8000)
        self.assertEqual(len(c2), 2000)



import numpy

from amuse.test import amusetest
from amuse.units import units

from amuse.ext.roche_radius import Roche_Orbit, sepinsky_formula


class TestRocheRadius(amusetest.TestCase):

    def test1(self):
        """ Test that the basic numbers from Sepinsky are not unreasonable. """
        self.assertAlmostEqual(sepinsky_formula(), 1.0, places=1)

        q_values = 10.0**numpy.arange(-8, 9)
        answers = [1.0] * len(q_values)
        self.assertAlmostEqual(sepinsky_formula(q=q_values), answers, places=1)

        a_values = 10.0**numpy.arange(-4, 2)
        answers = [1.0]*len(a_values)
        self.assertIsOfOrder(sepinsky_formula(q=1.0, A=a_values), answers)

        self.assertIsOfOrder(sepinsky_formula(q=1e-4, A=a_values), answers)

        self.assertIsOfOrder(sepinsky_formula(q=1e4, A=a_values), answers)

    def test2(self):
        """ Test the Roche_Orbit class """

        roche_orbit = Roche_Orbit()
        roche_orbit.semimajor_axis = 1 | units.AU

        print(roche_orbit.eggleton_roche_radius().as_quantity_in(units.RSun))
        self.assertAlmostEqual(roche_orbit.eggleton_roche_radius(), 81.5 | units.RSun, places=2)
        self.assertAlmostEqual(roche_orbit.sepinsky_roche_radius(), 81.32 | units.RSun, places=2)

        roche_orbit.eccentricity = 0.5

        self.assertAlmostEqual(roche_orbit.eggleton_roche_radius(), 40.75 | units.RSun, places=2)
        self.assertAlmostEqual(roche_orbit.sepinsky_roche_radius(), 39.62 | units.RSun, places=2)

        roche_orbit.semimajor_axis = 20.0 | units.AU
        roche_orbit.mass_2 = 10000.0 | units.MSun
        roche_orbit.eccentricity = 0.95
        roche_orbit.angular_velocity_ratio = 0.1

        self.assertAlmostEqual(roche_orbit.eggleton_roche_radius(), 4.87 | units.RSun, places=2)
        self.assertAlmostEqual(roche_orbit.sepinsky_roche_radius(), 5.39 | units.RSun, places=2)

        roche_orbit.eccentricity = numpy.array([0.0, 0.5, 0.95])
        self.assertAlmostEqual(roche_orbit.sepinsky_roche_radius(), [107.92, 53.93, 5.39] | units.RSun, places=2)



from amuse.test import amusetest
import os.path
import numpy
from amuse.io import phigrape
from amuse.units import units
from amuse.units import constants
from amuse.units import nbody_system
from amuse.units import quantities
from amuse import datamodel


class Test(amusetest.TestCase):

    def test(self):
        directory = os.path.dirname(__file__)
        instance = phigrape.Inp2Particles()
        instance.convert_to_particles(os.path.join(directory, 'plummer_100.ini'))
        rev_instance = phigrape.Particles2Inp()
        output = os.path.join(self.get_path_to_results(), 'plummer_back_100.ini')
        rev_instance.convert_to_inp(instance.Particles, output)

        control_instance = phigrape.Inp2Particles()
        control_instance.convert_to_particles(os.path.join(directory, output))

        self.assertAlmostEqual(control_instance.Particles.mass, instance.Particles.mass, 16)
        self.assertAlmostEqual(control_instance.Particles[1].position, instance.Particles[1].position, 16)
        self.assertAlmostEqual(control_instance.Particles[1].velocity, instance.Particles[1].velocity, 16)



import os.path
import numpy
from amuse.test.amusetest import get_path_to_results, TestWithMPI
try:
    from matplotlib import pyplot
    HAS_MATPLOTLIB = True
    from amuse.plot import plot, semilogy, xlabel, ylabel, loglog
except ImportError:
    HAS_MATPLOTLIB = False

from amuse.support.exceptions import AmuseException
from amuse.ext.grid_to_sph import Grid2SPH, convert_grid_to_SPH
from amuse.units import units
from amuse.units import generic_unit_system
from amuse.units import nbody_system
from amuse.units import constants
from amuse.units.generic_unit_converter import ConvertBetweenGenericAndSiUnits
from amuse.datamodel import Particles
from amuse.datamodel import Particle
from amuse.datamodel import ParticlesSuperset
from amuse.datamodel import Grid


def create_grid(*arg):
    grid = Grid.create(*arg)
    grid.add_vector_attribute("momentum", ["rhovx", "rhovy", "rhovz"])
    return grid


class TestGrid2SPH(TestWithMPI):

    def setup_simple_grid(self):
        test_grid = create_grid((4, 3, 2), [1.0, 1.0, 1.0] | units.m)
        test_grid.rho = numpy.linspace(1.0, 2.0, num=24).reshape(test_grid.shape) | units.kg/units.m**3
        test_grid.rhovx = test_grid.rho * (3.0 | units.m/units.s)
        test_grid.rhovy = test_grid.rho * (4.0 | units.m/units.s)
        test_grid.rhovz = test_grid.rho * (0.0 | units.m/units.s)
        test_grid.energy = test_grid.rho * ((1.0 | (units.m/units.s)**2) + 0.5 * (5.0 | units.m/units.s)**2)
        return test_grid

    def test0(self):
        print("Testing the simple example grid")
        test_grid = self.setup_simple_grid()
        self.assertEqual(test_grid.position[0][0][0],  [1.0/8.0, 1.0/6.0, 1.0/4.0] | units.m)
        self.assertEqual(test_grid.position[-1][-1][-1],  [7.0/8.0, 5.0/6.0, 3.0/4.0] | units.m)
        self.assertEqual(test_grid.momentum[0][0][0],  [3.0, 4.0, 0.0] | (units.kg/units.m**3) * (units.m/units.s))
        self.assertEqual(test_grid.momentum[-1][-1][-1],  [6.0, 8.0, 0.0] | (units.kg/units.m**3) * (units.m/units.s))
        self.assertEqual(test_grid.energy[0][0][0], 13.5 | (units.J/units.m**3))
        self.assertEqual(test_grid.energy[-1][-1][-1], 27.0 | (units.J/units.m**3))

    def test1(self):
        print("Testing the converter")
        number_of_particles = 10000
        test_grid = self.setup_simple_grid()
        converter = Grid2SPH(test_grid, number_of_particles)
        self.assertTrue(converter.grid is test_grid)
        self.assertEqual(converter.shape, (4, 3, 2))
        self.assertEqual(converter.number_of_sph_particles, number_of_particles)
        self.assertEqual(converter.base_distribution_type, "uniform")

        converter.setup_lookup_tables()
        converter.setup_variates()
        self.assertEqual(converter.cumulative_weight[0],  1.0/(1.5*4*3*2))
        self.assertEqual(converter.cumulative_weight[-1], 1.0)
        self.assertEqual(converter.position_lookup_table[0],  [1.0/8.0, 1.0/6.0, 1.0/4.0] | units.m)
        self.assertEqual(converter.position_lookup_table[-1], [7.0/8.0, 5.0/6.0, 3.0/4.0] | units.m)
        self.assertEqual(converter.position_lookup_table[9],  [3.0/8.0, 3.0/6.0, 3.0/4.0] | units.m)
        self.assertAlmostEqual(converter.velocity_lookup_table,  [3.0, 4.0, 0.0] | units.m/units.s)
        self.assertAlmostEqual(converter.specific_internal_energy_lookup_table,  1.0 | units.J/units.kg)
        self.assertEqual(converter.cellsize_unit,  units.m)
        self.assertTrue(converter.cellsize_unit is units.m)
        self.assertAlmostEqual(converter.cellsize_number, [0.25, 1/3.0, 0.5])
        self.assertAlmostEqual(converter.mass, 1.5 | units.kg)
        # The number of particles in a cell should scale with the amount of mass in the cell:
        self.assertAlmostRelativeEqual(
            converter.mass * numpy.histogram(converter.indices, bins=4*3*2)[0] * 1.0/number_of_particles,
            test_grid.rho.flatten()*test_grid.cellsize().prod(),
            places=2
        )

    def test2(self):
        print("Testing the user interface")
        number_of_particles = 10000
        test_grid = self.setup_simple_grid()
        sph_particles = convert_grid_to_SPH(test_grid, number_of_particles)
        self.assertEqual(len(sph_particles), number_of_particles)
        self.assertAlmostEqual(sph_particles.mass.sum(),  1.5 | units.kg)
        self.assertAlmostEqual(sph_particles.velocity,  [3.0, 4.0, 0.0] | units.m/units.s)
        self.assertAlmostEqual(sph_particles.u,  1.0 | (units.m/units.s)**2)
        # The number of particles in a cell should scale with the amount of mass in the cell:
        self.assertAlmostRelativeEqual(
            (1.5 | units.kg)/number_of_particles * numpy.histogramdd(
                sph_particles.position.value_in(units.m), bins=(4, 3, 2))[0],
            test_grid.rho*test_grid.cellsize().prod(),
            places=2
        )
        self.assertAlmostEqual(sph_particles.h_smooth,  (50.0/number_of_particles)**(1.0/3) | units.m)

    def test3(self):
        print("Testing the user interface, random base_distribution_type")
        number_of_particles = 10000
        test_grid = self.setup_simple_grid()
        sph_particles = convert_grid_to_SPH(test_grid, number_of_particles,
            base_distribution_type="random", seed=12345)
        self.assertEqual(len(sph_particles), number_of_particles)
        self.assertAlmostEqual(sph_particles.mass.sum(),  1.5 | units.kg)
        self.assertAlmostEqual(sph_particles.velocity,  [3.0, 4.0, 0.0] | units.m/units.s)
        self.assertAlmostEqual(sph_particles.u,  1.0 | (units.m/units.s)**2)
        # For 'random', the number of particles in a cell should scale only on average
        # with the amount of mass in the cell:
        self.assertAlmostRelativeEqual(
            ((1.5 | units.kg)/number_of_particles * numpy.histogramdd(
                sph_particles.position.value_in(units.m), bins=(4, 3, 2))[0]).sum(),
            (test_grid.rho*test_grid.cellsize().prod()).sum(),
            places=2
        )
        self.assertRaises(AssertionError,
            self.assertAlmostRelativeEqual,
                (1.5 | units.kg)/number_of_particles * numpy.histogramdd(sph_particles.position.value_in(units.m), bins=(4, 3, 2))[0],
                test_grid.rho*test_grid.cellsize().prod(),
                places=2,
        )
        self.assertAlmostEqual(sph_particles.h_smooth,  (50.0/number_of_particles)**(1.0/3) | units.m)

    def test4(self):
        print("Testing exceptions")
        number_of_particles = 10000
        test_grid = self.setup_simple_grid()
        self.assertEqual(test_grid[0].number_of_dimensions(), 2)
        self.assertRaises(AmuseException, convert_grid_to_SPH, test_grid[0], number_of_particles,
            expected_message="Grid must be 3D")
        self.assertRaises(AmuseException, convert_grid_to_SPH, test_grid,
            number_of_particles, base_distribution_type="bogus",
            expected_message="Unknown base_distribution_type: bogus. Possible "
                "options are: 'random' or 'uniform'.")



from amuse.test import amusetest
import os.path
import numpy
from amuse.units import units
from amuse.units import constants
from amuse.units import nbody_system
from amuse.units import quantities
from amuse.ic.plummer import new_plummer_model
from amuse.datamodel import rotation
from amuse.datamodel import particles


class TestRotations(amusetest.TestCase):

    def test01(self):
        x = rotation.new_rotation_matrix(0, 0, 0)
        self.assertAlmostRelativeEquals(x, ((1, 0, 0), (0, 1, 0), (0, 0, 1)))

        x = rotation.new_rotation_matrix(numpy.pi, 0, 0)
        print(x)
        self.assertAlmostRelativeEquals(x, ((1, 0, 0), (0, -1, 0), (0, 0, -1)))
        x = rotation.new_rotation_matrix(0, numpy.pi, 0)
        self.assertAlmostRelativeEquals(x, ((-1, 0, 0), (0, 1, 0), (0, 0, -1)))
        x = rotation.new_rotation_matrix(0, 0, numpy.pi)
        self.assertAlmostRelativeEquals(x, ((-1, 0, 0), (0, -1, 0), (0, 0, 1)))

    def test02(self):
        x = rotation.new_rotation_matrix(numpy.pi / 2.0, 0, 0)
        print(x)
        self.assertAlmostRelativeEquals(x, ((1, 0, 0), (0, 0, -1), (0, 1, 0)))
        x = rotation.new_rotation_matrix(0, numpy.pi / 2.0, 0)
        print(x)
        self.assertAlmostRelativeEquals(x, ((0, 0, 1), (0, 1, 0), (-1, 0, 0)))
        x = rotation.new_rotation_matrix(0, 0, numpy.pi / 2.0)
        print(x)
        self.assertAlmostRelativeEquals(x, ((0, -1, 0), (1, 0, 0), (0, 0, 1)))
        x = rotation.new_rotation_matrix(numpy.pi / 2.0, numpy.pi / 2.0, 0)
        print(x)
        self.assertAlmostRelativeEquals(x, ((0, 1, 0), (0, 0, -1), (-1, 0, 0)))

    def test03(self):
        positions = [[1.0, 2.0, 3.0]] | units.m
        rotated = rotation.rotated(positions, 0.0, 0.0, 0.0)

        print(rotated)

        self.assertAlmostRelativeEquals(rotated, [[1.0, 2.0, 3.0]] | units.m)
        rotated = rotation.rotated(positions, numpy.pi, 0.0, 0.0)

        print(rotated)

        self.assertAlmostRelativeEquals(rotated, [[1.0, -2.0, -3.0]] | units.m)

    def test04(self):
        positions = [[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]] | units.m
        rotated = rotation.rotated(positions, 0.0, 0.0, 0.0)

        print(rotated)

        self.assertAlmostRelativeEquals(rotated, [[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]] | units.m)

        rotated = rotation.rotated(positions, numpy.pi, 0.0, 0.0)
        print(rotated)

        self.assertAlmostRelativeEquals(rotated, [[1.0, -2.0, -3.0], [4.0, -5.0, -6.0]] | units.m)

    def test05(self):
        positions = [[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]] | units.m
        rotated = rotation.rotated(positions,  numpy.pi/2, 0.0, 0.0)

        print(rotated)

        self.assertAlmostRelativeEquals(rotated, [[1.0, -3.0, 2.0], [4.0, -6.0, 5.0]] | units.m)

    def test06(self):
        particles = new_plummer_model(100)
        kinetic_energy0 = particles.kinetic_energy()
        potential_energy0 = particles.potential_energy(G=nbody_system.G)
        particles.position = rotation.rotated(particles.position,  numpy.pi/2, 0.0, 0.0)
        particles.velocity = rotation.rotated(particles.velocity,  numpy.pi/2, 0.0, 0.0)

        kinetic_energy1 = particles.kinetic_energy()
        potential_energy1 = particles.potential_energy(G=nbody_system.G)
        self.assertAlmostRelativeEquals(kinetic_energy1, kinetic_energy0)
        self.assertAlmostRelativeEquals(potential_energy1, potential_energy0)

    def test07(self):
        particles = new_plummer_model(100)
        kinetic_energy0 = particles.kinetic_energy()
        potential_energy0 = particles.potential_energy(G=nbody_system.G)
        particles.position = rotation.rotated(particles.position,  numpy.pi/3, numpy.pi/2, 0.0)
        particles.velocity = rotation.rotated(particles.velocity,  numpy.pi/3, numpy.pi/2, 0.0)

        kinetic_energy1 = particles.kinetic_energy()
        potential_energy1 = particles.potential_energy(G=nbody_system.G)
        self.assertAlmostRelativeEquals(kinetic_energy1, kinetic_energy0)
        self.assertAlmostRelativeEquals(potential_energy1, potential_energy0)

    def test08(self):
        particles = new_plummer_model(100)
        kinetic_energy0 = particles.kinetic_energy()
        potential_energy0 = particles.potential_energy(G=nbody_system.G)

        particles.move_to_center()
        particles.position += [3, 0, 2] | nbody_system.length
        particles.rotate(numpy.pi/4, numpy.pi/2, 0.0)
        self.assertAlmostRelativeEquals(particles.center_of_mass(),
            [numpy.sqrt(2), -numpy.sqrt(2), -3] | nbody_system.length, 7)

        kinetic_energy1 = particles.kinetic_energy()
        potential_energy1 = particles.potential_energy(G=nbody_system.G)
        self.assertAlmostRelativeEquals(kinetic_energy1, kinetic_energy0)
        self.assertAlmostRelativeEquals(potential_energy1, potential_energy0)

    def test09(self):
        print("Test add_spin particle attribute, to add rigid body rotation")
        numpy.random.seed(123456)
        particles = new_plummer_model(1000)
        kinetic_energy0 = particles.kinetic_energy()
        potential_energy0 = particles.potential_energy(G=nbody_system.G)

        particles.position += [3, 0, 2] | nbody_system.length
        particles.velocity += [1, 10, 100] | nbody_system.speed
        particles.add_spin(3.0 | nbody_system.time**-1)

        self.assertAlmostRelativeEquals(particles.center_of_mass(), [3, 0, 2] | nbody_system.length, 12)
        self.assertAlmostRelativeEquals(particles.potential_energy(G=nbody_system.G), potential_energy0, 12)
        self.assertAlmostRelativeEquals(particles.center_of_mass_velocity(), [1, 10, 100] | nbody_system.speed, 12)

        r = particles.position - particles.center_of_mass()
        v = particles.velocity - particles.center_of_mass_velocity()
        spin_direction = (r).cross(v).mean(axis=0)
        spin_direction /= spin_direction.length()
        R = r - r*spin_direction
        omega = ((R).cross(v) / R.lengths_squared().reshape((-1, 1))).mean(axis=0).length()
        self.assertAlmostEqual(spin_direction, [0, 0, 1.0], 1)
        self.assertAlmostEqual(omega, 3.0 | nbody_system.time**-1, 1)

        particles.add_spin([1.0, 0, -3.0] | nbody_system.time**-1)
        v = particles.velocity - particles.center_of_mass_velocity()
        spin_direction = (r).cross(v).mean(axis=0)
        spin_direction /= spin_direction.length()
        R = r - r*spin_direction
        omega = ((R).cross(v) / R.lengths_squared().reshape((-1, 1))).mean(axis=0).length()
        self.assertAlmostEqual(omega, 1.0 | nbody_system.time**-1, 1)

    def test10(self):
        print("test conservation of dot, transformation of cross")
        p = particles.Particles(1)
        p.position = [1., 2., 3.]
        p.velocity = [-4, 5, 6.]

        dot1 = p[0].position.dot(p[0].velocity)
        cross1 = numpy.cross(p[0].position, p[0].velocity)

        rm = rotation.new_rotation_matrix(0.1, 0.5, 3.5)
        p.rotate(0.1, 0.5, 3.5)

        dot2 = p[0].position.dot(p[0].velocity)
        cross2 = numpy.cross(p[0].position, p[0].velocity)

        self.assertAlmostRelativeEquals(dot1, dot2)
        self.assertAlmostRelativeEquals(cross2, cross1.dot(numpy.linalg.inv(rm)))

    def test11(self):
        print("test conservation of dot, transformation of cross with units")
        p = particles.Particles(5)
        p.position = [1., 2., 3.] | units.km
        p.velocity = [-4, 5, 6.] | units.kms

        p[1:].position *= 0
        p[1:].velocity *= 0

        dot1 = p[0].position.dot(p[0].velocity)
        cross1 = p[0].position.cross(p[0].velocity)

        rm = rotation.new_rotation_matrix(0.1, 0.5, 3.5)
        p.rotate(0.1, 0.5, 3.5)

        dot2 = p[0].position.dot(p[0].velocity)
        cross2 = p[0].position.cross(p[0].velocity)

        self.assertAlmostRelativeEquals(dot1.value_in(units.km**2/units.s), dot2.value_in(units.km**2/units.s))
        self.assertAlmostRelativeEquals(cross2.value_in(units.km**2/units.s), cross1.dot(numpy.linalg.inv(rm)).value_in(units.km**2/units.s))




from amuse.test.amusetest import TestWithMPI

from amuse.community.interface import gd
from amuse.community.interface.gd import GravitationalDynamicsInterface
from amuse.rfi.tools import create_definition
from amuse.rfi.tools.create_definition import CodeDocStringProperty
from amuse.rfi.core import LegacyFunctionSpecification


class TestGravitationalDynamics(TestWithMPI):
    def test1(self):
        x = GravitationalDynamicsInterface()

        function = GravitationalDynamicsInterface.new_particle
        specification = function.specification
        self.assertTrue(specification.description.startswith("Define a new particle"))

    def test2(self):
        specification = LegacyFunctionSpecification()
        specification.name = 'test'
        specification.addParameter('one', 'd', specification.IN, 'first parameter')
        specification.description = 'Example function'

        x = create_definition.CreateDescriptionOfAFunctionSpecification()
        x.specification = specification
        x.start()

        self.assertTrue(x.out.string.find('void test(float64 one)') > 0)
        self.assertTrue(x.out.string.find('Example function') >= 0)
        self.assertTrue(x.out.string.find(':param one:') > 0)

    def test3(self):
        specification = LegacyFunctionSpecification()
        specification.name = 'test'
        specification.addParameter('one', 'd', specification.IN, 'first parameter')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'

        x = create_definition.CreateDescriptionOfAFunctionSpecification()
        x.specification = specification
        x.start()

        self.assertTrue(x.out.string.find('int32 test(float64 one)') > 0)
        self.assertTrue(x.out.string.find(':returns:') > 0)

    def test4(self):
        specification = LegacyFunctionSpecification()
        specification.name = 'test'
        specification.addParameter('one', 'd', specification.IN, 'first parameter')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'

        x = create_definition.CreateFortranStub()
        x.specification = specification
        x.start()

        self.assertTrue(x.out.string.find('FUNCTION test(one)') >= 0)
        self.assertTrue(x.out.string.find('END FUNCTION') > 0)
        self.assertTrue(x.out.string.find('DOUBLE PRECISION :: one') > 0)

    def test5(self):
        class WithLegacyDocStringProperty(object):

            def __init__(self):
                "orignal doc"
                pass

            @property
            def specification(self):
                specification = LegacyFunctionSpecification()
                specification.name = 'test'
                specification.addParameter('one', 'd', specification.IN, 'first parameter')
                specification.result_type = 'i'
                specification.result_doc = 'an integer'
                specification.description = 'Example function'
                return specification

            __doc__ = CodeDocStringProperty()

        self.assertEqual("orignal doc", WithLegacyDocStringProperty.__doc__)
        instance = WithLegacyDocStringProperty()
        instance_documentation = WithLegacyDocStringProperty().__doc__
        self.assertTrue(instance_documentation.find('FUNCTION test(one)') >= 0)

    def test6(self):
        print("Testing description of Legacy Function with output parameter")
        specification = LegacyFunctionSpecification()
        specification.name = 'test'
        specification.addParameter('one', 'd', specification.OUT, 'first parameter')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'

        x = create_definition.CreateDescriptionOfAFunctionSpecification()
        x.specification = specification
        x.start()
        self.assertTrue(x.out.string.find('int32 test(float64 * one)') > 0)
        self.assertTrue(x.out.string.find(':returns:') > 0)

    def test7(self):
        print("Testing __str__ of Legacy Function")
        specification = LegacyFunctionSpecification()
        specification.name = 'test'
        specification.addParameter('one', 'f', specification.IN, 'first parameter, type: float')
        specification.addParameter('two', 'd', specification.OUT, 'second parameter, type double')
        specification.result_type = 'i'
        specification.result_doc = 'an integer'
        specification.description = 'Example function'
        self.assertEqual(str(specification), "function: int test(float one)\noutput: double two, int __result")



import numpy.random

from amuse.test import amusetest
from amuse.units import units

from amuse.ic.brokenimf import new_kroupa_mass_distribution


class TestMassFunctions(amusetest.TestCase):

    def test_kroupa_default_masslimits(self):
        """
        Test the default mass limits of a Kroupa mass distribution
        """
        numpy.random.seed(124)
        mass_min = 0.01 | units.MSun
        mass_max = 125 | units.MSun
        mass = new_kroupa_mass_distribution(10000)
        self.assertTrue(mass.max() <= mass_max)
        self.assertTrue(mass.min() >= mass_min)

    def test_kroupa_lower_masslimit(self):
        """
        Test the lower mass limits of a Kroupa mass distribution
        """
        numpy.random.seed(124)
        mass_min = 1.0 | units.MSun
        mass = new_kroupa_mass_distribution(
            10000, mass_min=mass_min)
        self.assertTrue(mass.min() >= mass_min)

    def test_kroupa_upper_masslimit(self):
        """
        Test the upper mass limits of a Kroupa mass distribution
        """
        numpy.random.seed(124)

        mass_max = 1.0 | units.MSun
        mass = new_kroupa_mass_distribution(
            10000, mass_max=mass_max)
        self.assertTrue(mass.max() <= mass_max)

    def test_kroupa_masslimits(self):
        """
        Test both mass limits of a Kroupa mass distribution
        """
        numpy.random.seed(124)

        mass_max = 1.0 | units.MSun
        mass_min = 1.0 | units.MSun
        mass = new_kroupa_mass_distribution(
            10000, mass_min=mass_min, mass_max=mass_max)
        self.assertEqual(mass.max(), mass.min())



import numpy

from amuse.test.amusetest import TestWithMPI

from amuse.community.seba.interface import SeBaInterface, SeBa

from amuse.units import units
from amuse.units import constants
from amuse.datamodel import Particle
from amuse.datamodel import Particles


class TestSeBaInterface(TestWithMPI):

    def test1(self):
        instance = self.new_instance_of_an_optional_code(SeBaInterface)

        endtime, mass, radius, luminosity, temperature, time_step, stellar_type, error = instance.evolve_star(1, 4600, 0.02)
        self.assertEqual(error, 0)
        self.assertTrue(endtime <= 4600.0)
        self.assertAlmostRelativeEqual(endtime, 4600.0, 4)
        self.assertAlmostRelativeEqual(mass, 1.0, 6)
        self.assertAlmostRelativeEqual(radius, 0.9856, 4)
        self.assertAlmostRelativeEqual(luminosity, 0.9585, 4)
        self.assertAlmostRelativeEqual(temperature, 5751, 4)
        self.assertAlmostRelativeEqual(time_step, 1089.3, 4)
        self.assertEqual(stellar_type, 1)

        instance.stop()

    def test2(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle(1.)
        self.assertEqual(error, 0)
        self.assertEqual(index, 1)
        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 1.0, 6)
        value, error = instance.get_radius(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 0.88824945029751212, 6)

        stellar_type, error = instance.get_stellar_type(index)
        self.assertEqual(error, 0)
        self.assertEqual(stellar_type, 1)

        instance.stop()

    def test3(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle(1.)
        self.assertEqual(error, 0)
        self.assertEqual(index, 1)
        error = instance.evolve_model(4600)
        self.assertEqual(error, 0)

        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 1.0, 6)
        value, error = instance.get_radius(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 0.9856, 4)
        value, error = instance.get_temperature(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 5751, 4)
        value, error = instance.get_time_step(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 1089.3, 4)

        stellar_type, error = instance.get_stellar_type(index)
        self.assertEqual(error, 0)
        self.assertEqual(stellar_type, 1)

        instance.stop()

    def test4(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle(1.)
        self.assertEqual(error, 0)
        self.assertEqual(index, 1)
        for t in range(46):
            error = instance.evolve_model((t+1) * 100)
            self.assertEqual(error, 0)

        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 1.0, 6)
        value, error = instance.get_radius(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 0.9856, 4)
        value, error = instance.get_temperature(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 5751, 4)
        value, error = instance.get_time_step(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 1089.3, 4)

        stellar_type, error = instance.get_stellar_type(index)
        self.assertEqual(error, 0)
        self.assertEqual(stellar_type, 1)

        instance.stop()

    def test5(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle([1., 2., 3.])
        self.assertEqual(error, 0)
        self.assertEqual(index, [1, 2, 3])

        mass, error = instance.get_mass(2)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 2, 6)

        mass, error = instance.get_mass(3)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 3, 6)

        error = instance.evolve_model(4600)
        self.assertEqual(error, 0)

        mass, error = instance.get_mass(index)
        print(mass)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass[0], 1.0, 6)
        self.assertAlmostRelativeEqual(mass[1], 0.62973, 4)
        self.assertAlmostRelativeEqual(mass[2], 0.75012, 4)

        instance.stop()

    def test6(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle([1., 2., 3.])
        self.assertEqual(error, 0)
        self.assertEqual(index, [1, 2, 3])

        for t in range(46):
            error = instance.evolve_model((t+1) * 100)
            self.assertEqual(error, 0)

        mass, error = instance.get_mass(index)
        print(mass)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, [1.0, 0.62973, 0.75072], 4)

        instance.stop()

    def test7(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle([1., 2., 3.])
        self.assertEqual(error, 0)
        self.assertEqual(index, [1, 2, 3])

        mass, error = instance.get_mass(2)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 2, 6)

        mass, error = instance.get_mass(3)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 3, 6)

        mass, error = instance.get_mass(4)
        self.assertEqual(error, -1)

        error = instance.delete_star(2)
        self.assertEqual(error, 0)

        mass, error = instance.get_mass(2)
        self.assertEqual(error, -1)

        mass, error = instance.get_mass(3)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 3, 6)

        index, error = instance.new_particle(4.)
        self.assertEqual(error, 0)
        self.assertEqual(index, 4)

        instance.stop()

    def test8(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)

        index, error = instance.new_particle([3.0, 1.0, 2.0])
        self.assertEqual(error, 0)
        self.assertEqual(index, [1, 2, 3])

        error = instance.delete_star(1)
        self.assertEqual(error, 0)

        error = instance.evolve_model(4600)

        mass, error = instance.get_mass(2)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 1, 6)

        error = instance.delete_star(3)
        self.assertEqual(error, 0)

        index, error = instance.new_particle([5.0])
        self.assertEqual(error, 0)

        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 5.0, 6)
        error = instance.evolve_model(5000)

        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 0.99057, 4)

        error = instance.delete_star(2)
        self.assertEqual(error, 0)
        error = instance.delete_star(index)
        self.assertEqual(error, 0)

        for i in range(4):
            mass, error = instance.get_mass(index+1)
            self.assertEqual(error, -1)

        index, error = instance.new_particle([5.0])
        self.assertEqual(error, 0)

        error = instance.evolve_model(10000)

        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 0.99057, 4)

        instance.stop()

    def test9(self):
        instance = SeBaInterface()  # self.new_instance_of_an_optional_code(SeBaInterface)
        error = instance.initialize_code()
        self.assertEqual(error, 0)
        instance.set_metallicity(0.001)

        index, error = instance.new_particle([3.0, 0.3])
        self.assertEqual(error, 0)
        self.assertEqual(index, [1, 2])

        mu = (3.3 | units.MSun) * constants.G
        orbital_period = 200.0 | units.day
        semi_major_axis = (((orbital_period / 2.0 * numpy.pi)**2)*mu)**(1.0/3.0)
        print(semi_major_axis.value_in(units.RSun))

        eccentricity = 0.5
        index, error = instance.new_binary(
            semi_major_axis.value_in(units.RSun),
            eccentricity,
            index[0],
            index[1]
        )
        self.assertEqual(error, 0)
        self.assertEqual(index, 3)

        mass, error = instance.get_mass(index)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 3.3, 4)
        mass, error = instance.get_mass(2)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 0.3, 4)

        error = instance.evolve_model(300)
        self.assertEqual(error, 0)
        mass, error = instance.get_mass(1)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 2.98777, 4)
        mass, error = instance.get_mass(2)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 0.29999, 4)

        error = instance.evolve_model(400)
        self.assertEqual(error, 0)
        mass, error = instance.get_mass(1)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 0.902743, 4)
        mass, error = instance.get_mass(2)
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(mass, 0.3, 4)

        error = instance.delete_binary(index)
        self.assertEqual(error, 0)
        mass, error = instance.get_mass(index)
        self.assertEqual(error, -1)

        # check if singles are still in the mode and evolve
        value, error = instance.get_age([1, 2])
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 400, 4)
        error = instance.evolve_model(500)
        self.assertEqual(error, 0)
        value, error = instance.get_age([1, 2])
        self.assertEqual(error, 0)
        self.assertAlmostRelativeEqual(value, 500, 4)


class TestSeBa(TestWithMPI):

    def test_evolve_star(self):

        instance = self.new_instance_of_an_optional_code(SeBa)

        endtime, mass, radius, luminosity, temperature, time_step, stellar_type = instance.evolve_star(1 | units.MSun, 4600 | units.Myr, 0.02)

        self.assertTrue(endtime <= 4600 | units.Myr)
        self.assertAlmostRelativeEqual(mass, 1.0 | units.MSun, 4)
        self.assertAlmostRelativeEqual(radius, 0.9856 | units.RSun, 4)
        self.assertAlmostRelativeEqual(luminosity, 0.9585 | units.LSun, 4)
        self.assertAlmostRelativeEqual(temperature, 5751 | units.K, 4)
        self.assertAlmostRelativeEqual(time_step, 1089.3 | units.Myr, 4)
        self.assertEqual(stellar_type, 1 | units.stellar_type)

    def test_add_particle(self):
        instance = self.new_instance_of_an_optional_code(SeBa)

        p = Particle()
        p.mass = 5 | units.MSun
        p.metallicity = 0.02

        p = instance.particles.add_particle(p)
        instance.evolve_model(130 | units.Myr)
        print(p)

        self.assertAlmostRelativeEqual(p.mass, 0.9906 | units.MSun, 4)

    def test_evolution_of_close_binary_system(self):
        # print("Testing evolution of a close binary system...")
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.commit_parameters()
        stars = Particles(2)
        stars[0].mass = 3.0 | units.MSun
        stars[1].mass = 0.3 | units.MSun

        mu = (3.3 | units.MSun) * constants.G
        orbital_period = 200.0 | units.day
        semi_major_axis = (((orbital_period / (2.0 * numpy.pi))**2)*mu)**(1.0/3.0)

        instance.particles.add_particles(stars)

        binaries = Particles(1)

        binary = binaries[0]
        binary.semi_major_axis = semi_major_axis
        binary.eccentricity = 0.5
        binary.child1 = stars[0]
        binary.child2 = stars[1]

        instance.binaries.add_particles(binaries)

        from_seba_to_model = instance.particles.new_channel_to(stars)
        from_seba_to_model.copy()

        from_seba_to_model_binaries = instance.binaries.new_channel_to(binaries)
        from_seba_to_model_binaries.copy()

        previous_type = binary.child1.stellar_type
        results = []
        current_time = 0 | units.Myr

        while current_time < (480 | units.Myr):
            instance.update_time_steps()
            # The next line appears a bit weird, but saves time for this simple test.
            deltat = max(1.0*instance.binaries[0].time_step, 0.1 | units.Myr)
            current_time = current_time + deltat
            instance.evolve_model(current_time)
            from_seba_to_model.copy()
            from_seba_to_model_binaries.copy()
            if not binary.child1.stellar_type == previous_type:
                results.append((binary.age, binary.child1.mass, binary.child1.stellar_type))
                previous_type = binary.child1.stellar_type

        self.assertEqual(len(results), 6)
        for x in results:
            print(x)

        types = (
            "Hertzsprung Gap",
            "First Giant Branch",
            "Core Helium Burning",
            "First Asymptotic Giant Branch",
            "Giant Branch Naked Helium star",
            "Carbon/Oxygen White Dwarf",
        )

        for result, expected in zip(results, types):
            self.assertEqual(str(result[2]), expected)

        times = (
            377.6369 | units.Myr,
            379.8877 | units.Myr,
            382.3112 | units.Myr,
            473.4804 | units.Myr,
            475.4766 | units.Myr,
            476.6182 | units.Myr,
        )
        for result, expected in zip(results, times):
            self.assertAlmostEqual(result[0].value_in(units.Myr), expected.value_in(units.Myr), 0)

        masses = (
            3.0000 | units.MSun,
            3.0000 | units.MSun,
            2.9983 | units.MSun,
            2.9741 | units.MSun,
            0.6710 | units.MSun,
            0.6596 | units.MSun,
        )
        for result, expected in zip(results, masses):
            self.assertAlmostEqual(result[1].value_in(units.MSun), expected.value_in(units.MSun), 2)

        instance.stop()

    def test_set_parameter_metallicity(self):
        instance = self.new_instance_of_an_optional_code(SeBa)
        self.assertAlmostRelativeEquals(instance.parameters.metallicity, 0.02)
        instance.parameters.metallicity = 0.04
        self.assertAlmostRelativeEquals(instance.parameters.metallicity, 0.04)

    def test_set_parameter_logging(self):
        instance = self.new_instance_of_an_optional_code(SeBa)
        self.assertFalse(instance.parameters.is_logging_of_evolve_enabled)
        instance.parameters.is_logging_of_evolve_enabled = True
        self.assertTrue(instance.parameters.is_logging_of_evolve_enabled)

    def test_add_binary_particles(self):
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.commit_parameters()
        stars = Particles(2)
        stars[0].mass = 3.0 | units.MSun
        stars[1].mass = 0.3 | units.MSun

        mu = (3.3 | units.MSun) * constants.G
        orbital_period = 200.0 | units.day
        semi_major_axis = (((orbital_period / (2.0 * numpy.pi))**2)*mu)**(1.0/3.0)

        instance.particles.add_particles(stars)

        binaries = Particles(1)

        binary = binaries[0]
        binary.semi_major_axis = semi_major_axis
        binary.eccentricity = 0.5
        binary.child1 = stars[0]
        binary.child2 = stars[1]

        instance.binaries.add_particles(binaries)

        self.assertAlmostRelativeEquals(instance.binaries[0].child1.mass, 3.0 | units.MSun, 4)
        self.assertAlmostRelativeEquals(instance.binaries[0].child2.mass, 0.3 | units.MSun, 4)

    def xtest7(self):
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.parameters.metallicity = 0.03
        p = Particle()
        p.mass = 99.1605930967 | units.MSun

        p = instance.particles.add_particle(p)
        instance.evolve_model(614 | units.Myr)
        print(p.stellar_type)
        self.assertEqual(str(p.stellar_type), 'Black Hole')
        self.assertAlmostRelativeEqual(p.mass, 0.9906 | units.MSun, 4)

    def test_set_semi_major_axis_eccentricity(self):
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.parameters.supernova_kick_velocity = 0 | units.kms
        instance.commit_parameters()
        print("v_kick=", instance.parameters.supernova_kick_velocity)
        stars = Particles(2)
        stars[0].mass = 10 | units.MSun
        stars[1].mass = 9 | units.MSun

        semi_major_axis = 10000 | units.AU

        instance.particles.add_particles(stars)

        binaries = Particles(1)

        binary = binaries[0]
        binary.semi_major_axis = semi_major_axis
        binary.eccentricity = 0
        binary.child1 = stars[0]
        binary.child2 = stars[1]

        instance.binaries.add_particles(binaries)
        instance.evolve_model(30 | units.Myr)
        print(instance.particles)
        print(instance.binaries)

        self.assertAlmostRelativeEquals(instance.binaries[0].eccentricity, 1, 4)

    def test_add_stars_at_different_times(self):
        instance = self.new_instance_of_an_optional_code(SeBa)
        stars = Particles(2)
        stars[0].mass = 10.0 | units.MSun
        stars[1].mass = 9 | units.MSun

        instance.particles.add_particles(stars)
        instance.evolve_model(30 | units.Myr)

        self.assertAlmostRelativeEquals(instance.particles.age, [30, 30] | units.Myr)
        self.assertAlmostRelativeEquals(instance.model_time, 30 | units.Myr)
        self.assertAlmostRelativeEquals(instance.particles[0].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEquals(instance.particles[1].mass, 8.9351 | units.MSun, 4)
        stars = Particles(2)
        stars[0].mass = 10.0 | units.MSun
        stars[1].mass = 9 | units.MSun

        instance.particles.add_particles(stars)
        instance.evolve_model(60 | units.Myr)
        print(instance.particles.age)
        print(instance.particles.mass)
        self.assertAlmostRelativeEquals(instance.model_time, 60 | units.Myr)
        self.assertAlmostRelativeEquals(instance.particles.age, [60, 60, 30, 30] | units.Myr)
        self.assertAlmostRelativeEquals(instance.particles[2].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEquals(instance.particles[3].mass, 8.9351 | units.MSun, 4)

    def test_supernova_stopping_condition(self):
        """ Test supernova stopping condition """
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.stopping_conditions.supernova_detection.enable()

        p = Particle()
        p.mass = 10 | units.MSun
        p.metallicity = 0.02

        p = instance.particles.add_particle(p)
        instance.set_supernova_kick_velocity(0.0 | units.kms)
        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), True)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, p.key)

        self.assertAlmostRelativeEqual(p.age, 27.2715 | units.Myr, 4)

        self.assertAlmostRelativeEqual(p.mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(p.natal_kick_velocity, [-54.070, 12.818, 42.583] | units.kms, 4)

    def test_supernova_stopping_condition_in_a_binary(self):
        """ Test supernova stopping condition in a binary """
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.stopping_conditions.supernova_detection.enable()

        stars = Particles(2)
        stars[0].mass = 10.0 | units.MSun
        stars[1].mass = 5.0 | units.MSun
        instance.particles.add_particles(stars)

        binaries = Particles(1)
        binary = binaries[0]
        binary.semi_major_axis = 1.e6 | units.RSun
        binary.eccentricity = 0.1
        binary.child1 = stars[0]
        binary.child2 = stars[1]
        instance.binaries.add_particles(binaries)

        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), True)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, instance.binaries[0].child1.key)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, instance.particles[0].key)

        print(instance.parameters)
        self.assertAlmostRelativeEqual(instance.particles[0].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[0].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].mass, 5.0 | units.MSun, 4)
#        self.assertAlmostRelativeEqual(instance.particles[0].natal_kick_velocity, [0,0,0] | units.kms, 4)

        self.assertAlmostRelativeEqual(instance.binaries[0].child1.age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.binaries[0].child2.age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.binaries[0].child1.mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.binaries[0].child2.mass, 5.0 | units.MSun, 4)
#        self.assertAlmostRelativeEqual(instance.binaries[0].child1.natal_kick_velocity, [0,0,0] | units.kms, 4)

    def test_supernova_stopping_condition_with_multiple_stars(self):
        """ Test supernova stopping condition with multiple stars """
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.stopping_conditions.supernova_detection.enable()

        stars = Particles(3)
        stars[0].mass = 10.0 | units.MSun
        stars[1].mass = 5.0 | units.MSun
        stars[2].mass = 0.5 | units.MSun
        instance.particles.add_particles(stars)

        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), True)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, instance.particles[0].key)

        self.assertAlmostRelativeEqual(instance.particles[0].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[0].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].mass, 5.0 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].mass, 0.5 | units.MSun, 4)
#        self.assertAlmostRelativeEqual(instance.particles[0].natal_kick_velocity, [0,0,0] | units.kms, 4)

    def test_supernova_stopping_condition_with_multiple_stars_multiple_supernovae(self):
        """ Test supernova stopping condition with multiple stars """
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.stopping_conditions.supernova_detection.enable()

        stars = Particles(3)
        stars[0].mass = 10.0 | units.MSun
        stars[1].mass = 11.0 | units.MSun
        stars[2].mass = 0.5 | units.MSun
        instance.particles.add_particles(stars)

        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), True)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, instance.particles[1].key)

        self.assertAlmostRelativeEqual(instance.particles[0].age, 23.02102 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].age, 23.02102 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].age, 23.02102 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[0].mass, 9.974497 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].mass, 1.29259 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].mass, 0.5 | units.MSun, 4)
#        self.assertAlmostRelativeEqual(instance.particles[0].natal_kick_velocity, [0,0,0] | units.kms, 4)

        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), True)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, instance.particles[0].key)

        self.assertAlmostRelativeEqual(instance.particles[0].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[0].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].mass, 1.29259 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].mass, 0.5 | units.MSun, 4)
#        self.assertAlmostRelativeEqual(instance.particles[0].natal_kick_velocity, [0,0,0] | units.kms, 4)

    def test_supernova_stopping_condition_with_multiple_stars_of_equal_mass(self):
        """ Test supernova stopping condition with multiple stars of equal mass """
        instance = self.new_instance_of_an_optional_code(SeBa)
        instance.stopping_conditions.supernova_detection.enable()

        stars = Particles(3)
        stars[0].mass = 10.0 | units.MSun
        stars[1].mass = 10.0 | units.MSun
        stars[2].mass = 0.5 | units.MSun
        instance.particles.add_particles(stars)

        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), True)
        self.assertEqual(len(instance.stopping_conditions.supernova_detection.particles(0)), 2)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[0].key, instance.particles[0].key)
        self.assertEqual(instance.stopping_conditions.supernova_detection.particles(0)[1].key, instance.particles[1].key)

        self.assertAlmostRelativeEqual(instance.particles[0].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].age, 27.2715 | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[0].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].mass, 0.5 | units.MSun, 4)

#        self.assertAlmostRelativeEqual(instance.particles[0].natal_kick_velocity, [0,0,0] | units.kms, 4)
#        self.assertAlmostRelativeEqual(instance.particles[1].natal_kick_velocity, [0,0,0] | units.kms, 4)

        instance.evolve_model(30 | units.Myr)
        self.assertEqual(instance.stopping_conditions.supernova_detection.is_set(), False)

        self.assertAlmostRelativeEqual(instance.particles[0].age, 30. | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].age, 30. | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].age, 30. | units.Myr, 4)
        self.assertAlmostRelativeEqual(instance.particles[0].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[1].mass, 1.2507 | units.MSun, 4)
        self.assertAlmostRelativeEqual(instance.particles[2].mass, 0.5 | units.MSun, 4)



from amuse.support.interface import InCodeComponentImplementation

from amuse.test.amusetest import TestWithMPI
from amuse.test import compile_tools
from amuse.support import exceptions

import subprocess
import os
import time
from amuse.units import nbody_system
from amuse.units import units
from amuse import datamodel
from amuse.rfi.tools import create_c
from amuse.rfi import channel
from amuse.rfi.core import *
codestring = """
#include <stdio.h>
#include <stdlib.h>

double grid0;
double grid1[10];
double grid2[10][10];

int set0(double in) {
    grid0=in;
    return 0;
}
int get0(double *out) {
    *out=grid0;
    return 0;
}
int get_grid0_range() {
    return 0;
}

"""


class ForTestingInterface(CodeInterface):
    include_headers = ['worker_code.h']

    def __init__(self, exefile, **options):
        CodeInterface.__init__(self, exefile, **options)

    @remote_function
    def set0(a=0. | units.m):
        returns()

    @remote_function
    def get0():
        returns(a=0. | units.m)

    @remote_function
    def get_grid0_range():
        returns()


class ForTesting(InCodeComponentImplementation):

    def __init__(self, exefile, **options):
        InCodeComponentImplementation.__init__(self, ForTestingInterface(exefile, **options), **options)

    def define_grids(self, object):
        object.define_grid('grid0', grid_class=datamodel.RectilinearGrid, state_guard="before_new_set_instance")
        object.set_grid_range('grid0', 'get_grid0_range')
        object.add_getter('grid0', 'get0',  names=['a'])
        object.add_setter('grid0', 'set0',  names=['a'])


class TestCImplementationInterface(TestWithMPI):

    def build_worker(self):

        path = os.path.abspath(self.get_path_to_results())
        codefile = os.path.join(path, "code.o")
        headerfile = os.path.join(path, "worker_code.h")
        interfacefile = os.path.join(path, "interface.o")
        self.exefile = os.path.join(path, "c_worker")

        compile_tools.c_compile(codefile, codestring)

        uc = create_c.GenerateACHeaderStringFromASpecificationClass()
        uc.specification_class = ForTestingInterface
        uc.needs_mpi = False
        header = uc.result

        with open(headerfile, "w") as f:
            f.write(header)

        uc = create_c.GenerateACSourcecodeStringFromASpecificationClass()
        uc.specification_class = ForTestingInterface
        uc.needs_mpi = False
        code = uc.result

        compile_tools.cxx_compile(interfacefile, code, extra_args=['-I', path])
        compile_tools.c_build(self.exefile, [interfacefile, codefile])

    def setUp(self):
        super(TestCImplementationInterface, self).setUp()
        print("building...", end=' ')
        self.check_can_compile_modules()
        try:
            self.build_worker()
        except Exception as ex:
            print(ex)
            raise
        print("done")

    def test1(self):
        instance = ForTestingInterface(self.exefile)
        error = instance.set0(1)
        a_out, error = instance.get0()
        instance.stop()
        self.assertEqual(a_out, 1)
        self.assertEqual(error, 0)

    def test2(self):
        instance = ForTesting(self.exefile)
        print(instance.grid0)
        instance.grid0.a = 12. | units.m
        self.assertEqual(instance.grid0.a, 12. | units.m)
        instance.stop()

    def test3(self):
        instance1 = ForTesting(self.exefile)
        instance2 = ForTesting(self.exefile)
        instance1.grid0.a = 12. | units.m
        instance1.grid0.new_channel_to(instance2.grid0).copy_all_attributes()
        self.assertEqual(instance2.grid0.a, 12. | units.m)
        instance1.stop()
        instance2.stop()



from amuse.test import amusetest
from amuse.units import units, nbody_system, constants
from amuse.ic.isotropic_cloud import new_isotropic_cloud
from amuse.ext.orbital_elements import orbital_elements_from_binary
from amuse.datamodel import Particles


class TestSphericalIsotropicCloud(amusetest.TestCase):

    def test1(self):
        cloud = new_isotropic_cloud(2, m_cloud=0. | units.MSun)
        self.assertEqual(len(cloud), 2)
        self.assertEqual(cloud.mass.sum(), 0. | units.MSun)

    def test2(self):
        m_star = 0.666 | units.MSun
        a_min = 666. | units.AU
        a_max = 6666. | units.AU
        q_min = 6. | units.AU
        cloud = new_isotropic_cloud(66,
                                    m_star=m_star,
                                    a_min=a_min,
                                    a_max=a_max,
                                    q_min=q_min)
        binary = Particles(1)
        binary[0].mass = m_star
        binary[0].position = (0., 0., 0.) | units.AU
        binary[0].velocity = (0., 0., 0.) | units.kms
        for comet in cloud:
            binary.add_particle(comet)
            mass1, mass2, semimajor_axis, eccentricity, true_anomaly, inclination, long_asc_node, arg_per = \
              orbital_elements_from_binary(binary, G=constants.G)
            print(mass1, mass2, semimajor_axis, eccentricity, true_anomaly, inclination, long_asc_node, arg_per)
            self.assertTrue(a_min < semimajor_axis < a_max)
            self.assertTrue(q_min < semimajor_axis*(1.-eccentricity))
            binary.remove_particle(comet)



import sys
import os
import os.path
import shutil
import numpy
from subprocess import call

from amuse.community.sse.interface import SSEInterface, SSE

from amuse.test.amusetest import get_path_to_results, TestWithMPI
from amuse import io
from amuse.units import units
from amuse.datamodel import Particle, Particles
from amuse.ic.salpeter import new_salpeter_mass_distribution


class TestMPIInterface(TestWithMPI):

    class state(object):
        def __init__(self):
            self.stellar_type = 0.0
            self.zams_mass = 0.0
            self.mass = 0.0
            self.radius = 0.0
            self.luminosity = 0.0
            self.core_mass = 0.0
            self.core_radius = 0.0
            self.envelope_mass = 0.0
            self.envelope_radius = 0.0
            self.spin = 0.0
            self.epoch = 0.0
            self.t_ms = 0.0
            self.sse_age = 0.0

    def initialize_module_with_default_parameters(self, sse):
        metallicity = 0.02

        neta = 0.5
        bwind = 0.0
        hewind = 0.5
        sigma = 190.0

        ifflag = 0
        wdflag = 1
        bhflag = 0
        nsflag = 1
        mxns = 3.0

        pts1 = 0.05
        pts2 = 0.01
        pts3 = 0.02

        status = sse.initialize(metallicity,
            neta, bwind, hewind, sigma,
            ifflag, wdflag, bhflag, nsflag, mxns,
            pts1, pts2, pts3)

    def test1(self):
        sse = SSEInterface()

        metallicity = 0.02

        neta = 0.5
        bwind = 0.0
        hewind = 0.5
        sigma = 190.0

        ifflag = 0
        wdflag = 1
        bhflag = 0
        nsflag = 1
        mxns = 3.0

        pts1 = 0.05
        pts2 = 0.01
        pts3 = 0.02

        status = sse.initialize(metallicity,
            neta, bwind, hewind, sigma,
            ifflag, wdflag, bhflag, nsflag, mxns,
            pts1, pts2, pts3)

        self.assertEqual(status, 0)

        sse.stop()

    def test2(self):
        sse = SSEInterface()

        metallicity = 0.02

        neta = 0.5
        bwind = 0.0
        hewind = 0.5
        sigma = 190.0

        ifflag = 0
        wdflag = 1
        bhflag = 0
        nsflag = 1
        mxns = 3.0

        pts1 = 0.05
        pts2 = 0.01
        pts3 = 0.02

        status = sse.initialize(metallicity,
            neta, bwind, hewind, sigma,
            ifflag, wdflag, bhflag, nsflag, mxns,
            pts1, pts2, pts3)
        self.assertEqual(status, 0)
        new_state = self.state()
        new_state.mass = 1.0
        new_state.zams_mass = 1.0
        new_state.stellar_type = 1.0
        new_state.age = 1e-06
        result = sse.evolve_star(
            new_state.stellar_type,
            new_state.zams_mass, new_state.mass, new_state.radius,
            new_state.luminosity, new_state.core_mass, new_state.core_radius,
            new_state.envelope_mass, new_state.envelope_radius, new_state.spin,
            new_state.epoch, new_state.t_ms, new_state.sse_age, new_state.age
        )
        updated_state = self.state()
        (updated_state.stellar_type, updated_state.zams_mass, updated_state.mass, updated_state.radius,
            updated_state.luminosity, updated_state.core_mass, updated_state.core_radius,
            updated_state.envelope_mass, updated_state.envelope_radius, updated_state.spin,
            updated_state.epoch, updated_state.t_ms, updated_state.sse_age, updated_state.age) = result
        attributes = ('stellar_type', 'zams_mass', 'mass', 'radius', 'luminosity', 'core_mass', 'core_radius',
            'envelope_mass', 'envelope_radius', 'spin', 'epoch', 't_ms', 'sse_age', 'age')

        expected = {
            'zams_mass': '0x1.0000000000000p+0',
            'mass': '0x1.0000000000000p+0',
            'radius': '0x1.c6c8a1c793bcep-1',
            'luminosity': '0x1.653b1b2d0333bp-1',
            'core_mass': '0x0.0p+0',
            'core_radius': '0x0.0p+0',
            'envelope_mass': '0x1.0d6fc100ab510p-5',
            'envelope_radius': '0x1.db27631ba0e5ap-3',
            'spin': '0x1.07413b0522d63p+10',
            'epoch': '0x0.0p+0',
            't_ms': '0x1.57d90abe54643p+13',
            'sse_age': '0x1.0c6f7a0b5ed8dp-20',
            'age': '0x1.0c6f7a0b5ed8dp-20',
        }

        for x in expected:
            self.assertAlmostRelativeEqual(float.fromhex(expected[x]), getattr(updated_state, x))

        self.assertEqual(updated_state.age, 1e-06)
        dt = sse.get_time_step(updated_state.stellar_type,
            updated_state.zams_mass,
            updated_state.age,
            updated_state.mass,
            updated_state.t_ms,
            updated_state.epoch)
        self.assertAlmostEqual(dt, 550.1565, 2)
        sse.stop()

    def test3(self):
        sse = SSEInterface()
        self.initialize_module_with_default_parameters(sse)
        types = [1, 1, 1]
        masses = [10, 5, 4]
        radii = [5.0, 2.0, 1.0]
        luminosity = core_mass = core_radius = envelope_mass =\
        envelope_radius = spin = epoch = t_ms = [0.0, 0.0, 0.0]
        sse_age = age = [1e-6, 1e-06, 1e-6]
        result = sse.evolve_star(
            types,
            masses,
            masses,
            radii,
            luminosity,
            core_mass,
            core_radius,
            envelope_mass,
            envelope_radius,
            spin,
            epoch,
            t_ms,
            sse_age,
            age
        )
        self.assertEqual(result['mass'][0], 10)
        self.assertEqual(result['mass'][1], 5)
        self.assertAlmostEqual(result['mass'][2], 4.0, 2)
        sse.stop()

    def test4(self):
        sse = SSEInterface()
        self.initialize_module_with_default_parameters(sse)
        types = [1 for x in range(1, 4000)]
        masses = [1.0 + ((x / 4000.0) * 10.0) for x in range(1, 4000)]
        radii = [1.0 for x in range(1, 4000)]
        luminosity = core_mass = core_radius = envelope_mass =\
        envelope_radius = spin = epoch =\
        t_ms = [0.0 for x in range(1, 4000)]

        sse_age = age = [1e-06 for x in range(1, 4000)]
        result = sse.evolve_star(
            types,
            masses,
            masses,
            radii,
            luminosity,
            core_mass,
            core_radius,
            envelope_mass,
            envelope_radius,
            spin,
            epoch,
            t_ms,
            sse_age,
            age
        )
        self.assertEqual(len(result['mass']), 3999)
        sse.stop()


class TestSSE(TestWithMPI):

    def test1(self):
        sse = SSE()
        sse.commit_parameters()
        stars = Particles(1)
        star = stars[0]
        star.mass = 5 | units.MSun
        star.radius = 0.0 | units.RSun

        sse.particles.add_particles(stars)
        from_sse_to_model = sse.particles.new_channel_to(stars)
        from_sse_to_model.copy()

        previous_type = star.stellar_type
        results = []
        t0 = 0 | units.Myr
        current_time = t0

        while current_time < (125 | units.Myr):
            sse.update_time_steps()

            current_time = current_time + sse.particles[0].time_step

            sse.evolve_model(current_time)

            from_sse_to_model.copy()

            if not star.stellar_type == previous_type:
                results.append((star.age, star.mass, star.stellar_type))
                previous_type = star.stellar_type

        self.assertEqual(len(results), 6)

        times = (
            104.0 | units.Myr,
            104.4 | units.Myr,
            104.7 | units.Myr,
            120.1 | units.Myr,
            120.9 | units.Myr,
            121.5 | units.Myr
        )
        for result, expected in zip(results, times):
            self.assertAlmostEqual(result[0].value_in(units.Myr), expected.value_in(units.Myr), 1)

        masses = (
            5.000 | units.MSun,
            5.000 | units.MSun,
            4.998 | units.MSun,
            4.932 | units.MSun,
            4.895 | units.MSun,
            0.997 | units.MSun
        )
        for result, expected in zip(results, masses):
            self.assertAlmostEqual(result[1].value_in(units.MSun), expected.value_in(units.MSun), 3)

        types = (
            "Hertzsprung Gap",
            "First Giant Branch",
            "Core Helium Burning",
            "First Asymptotic Giant Branch",
            "Second Asymptotic Giant Branch",
            "Carbon/Oxygen White Dwarf",
        )

        for result, expected in zip(results, types):
            self.assertEqual(str(result[2]), expected)

        sse.stop()

    def test2(self):
        sse = SSE()
        sse.commit_parameters()
        stars = Particles(1)

        star = stars[0]
        star.mass = 5 | units.MSun
        star.radius = 0.0 | units.RSun

        sse.particles.add_particles(stars)
        sse.evolve_model(120.1 | units.Myr)

        self.assertAlmostEqual(sse.particles[0].mass.value_in(units.MSun), 4.932, 3)
        self.assertAlmostEqual(sse.particles[0].temperature.value_in(units.K), 4221., 0)

        sse.stop()

    def test3(self):
        sse = SSE()
        sse.commit_parameters()
        stars = Particles(1)

        star = stars[0]
        star.mass = 5 | units.MSun
        star.radius = 0.0 | units.RSun

        stars.synchronize_to(sse.particles)

        channel = sse.particles.new_channel_to(stars)
        channel.copy_attributes(sse.particles.get_attribute_names_defined_in_store())

        previous_type = sse.particles.stellar_type
        results = []

        sse.evolve_model(121.5 | units.Myr)

        channel.copy_attributes(sse.particles.get_attribute_names_defined_in_store())

        self.assertAlmostEqual(star.mass.value_in(units.MSun), 0.997, 3)

        sse.stop()

    def test5(self):
        sse = SSE()
        sse.commit_parameters()
        stars = Particles(1)

        star = stars[0]
        star.mass = 35 | units.MSun
        star.radius = 0.0 | units.RSun

        stars.synchronize_to(sse.particles)

        channel = sse.particles.new_channel_to(stars)
        channel.copy_attributes(sse.particles.get_attribute_names_defined_in_store())

        previous_type = star.stellar_type
        results = []

        dt = 1 | units.Myr
        t = 0 | units.Myr
        while t < 30 | units.Myr:
            t += dt
            sse.evolve_model(t)

        self.assertTrue(sse.particles[0].mass.value_in(units.MSun) < 10.6)

        sse.stop()

    def test6(self):
        print("Test whether a set of stars evolves synchronously...")
#       Create an array of stars with a range in stellar mass
        masses = [.5, 1., 2., 5., 10., 30.] | units.MSun
        number_of_stars = len(masses)
        stars = Particles(number_of_stars)
        stars.mass = masses

#       Initialize stellar evolution code
        instance = SSE()
        instance.commit_parameters()
        instance.particles.add_particles(stars)
        instance.commit_particles()

        from_code_to_model = instance.particles.new_channel_to(stars)
        from_code_to_model.copy()

        instance.evolve_model(end_time=125 | units.Myr)
        from_code_to_model.copy()

        end_types = (
            "deeply or fully convective low mass MS star",
            "Main Sequence star",
            "Main Sequence star",
            "Carbon/Oxygen White Dwarf",
            "Neutron Star",
            "Black Hole",
        )
        for i in range(number_of_stars):
            self.assertAlmostEqual(stars[i].age, 125.0 | units.Myr)
            self.assertTrue(stars[i].mass <= masses[i])
            self.assertEqual(str(stars[i].stellar_type), end_types[i])
        instance.stop()

    def test7(self):
        print("Test: evolve particles one at a time.")
        print("Used to be problematic, since initial_mass of idle particle is set to zero.")
        stars = Particles(2)
        stars.mass = 1.0 | units.MSun
        for star in stars:
            stellar_evolution = SSE()
            stellar_evolution.commit_parameters()
            stellar_evolution.particles.add_particles(star.as_set())
            stellar_evolution.commit_particles()
            from_stellar_evolution_to_model = stellar_evolution.particles.new_channel_to(star.as_set())
            stellar_evolution.evolve_model()
            from_stellar_evolution_to_model.copy()
            stellar_evolution.stop()
        self.assertEqual(stars[0].initial_mass, stars[1].initial_mass)
        self.assertEqual(stars[0].luminosity, stars[1].luminosity)
        self.assertEqual(stars[0].age, stars[1].age)
        print("Solved: SSE_muse_interface.f sets initial_mass to mass when necessary.")

    def test8(self):
        instance = SSE()
        self.assertEqual(instance.parameters.reimers_mass_loss_coefficient, 0.5)
        myvalue = 0.7
        instance.parameters.reimers_mass_loss_coefficient = myvalue
        self.assertEqual(instance.parameters.reimers_mass_loss_coefficient, myvalue)
        instance.commit_parameters()
        self.assertEqual(instance.parameters.reimers_mass_loss_coefficient, myvalue)
        instance.stop()

        instance = SSE()
        self.assertEqual(instance.parameters.reimers_mass_loss_coefficient, 0.5)
        myvalue = 0.7
        instance.parameters.reimers_mass_loss_coefficient = myvalue
        instance.parameters.set_defaults()
        instance.commit_parameters()
        self.assertEqual(instance.parameters.reimers_mass_loss_coefficient, 0.5)
        instance.stop()

    def test9(self):
        print("Test: large number of particles")
        stellar_evolution = SSE(max_message_length=500)
        stellar_evolution.commit_parameters()
        number_of_particles = 10000
        print("Has been tested with up to a million particles!")
        print("Now using ", number_of_particles, "particles only, for speed.")
        stars = Particles(number_of_particles)
        stars.mass = 1.0 | units.MSun
        stellar_evolution.particles.add_particles(stars)
        self.assertEqual(len(stellar_evolution.particles), number_of_particles)
        stellar_evolution.stop()

    def test10(self):
        stellar_evolution = SSE()
        stellar_evolution.commit_parameters()
        stars = Particles(10)
        stars.mass = 1.0 | units.MSun
        stellar_evolution.particles.add_particles(stars)
        self.assertEqual(stellar_evolution.particles._factory_for_new_collection(), Particles)

        filename = os.path.join(get_path_to_results(), "test.h5")
        if os.path.exists(filename):
            os.remove(filename)

        io.write_set_to_file(stellar_evolution.particles, filename, 'hdf5')
        stored_stars = io.read_set_from_file(filename, 'hdf5')
        self.assertEqual(len(stars), len(stored_stars))

        self.assertAlmostRelativeEquals(stars.mass, stored_stars.mass)

    def test11(self):
        print("Test evolve_model optional arguments: end_time and keep_synchronous")
        stars = Particles(3)
        stars.mass = [1.0, 2.0, 3.0] | units.MSun
        instance = SSE()
        instance.commit_parameters()
        instance.particles.add_particles(stars)

        self.assertEqual(instance.particles.age, [0.0, 0.0, 0.0] | units.yr)
        self.assertAlmostEqual(instance.particles.time_step, [550.1565, 58.2081, 18.8768] | units.Myr, 3)
        self.assertAlmostEqual(instance.particles.radius, [0.8882494502, 1.610210385, 1.979134445] | units.RSun)

        print("evolve_model without arguments: use shared timestep = min(particles.time_step)")
        instance.evolve_model()
        self.assertAlmostEqual(instance.particles.age, [18.8768, 18.8768, 18.8768] | units.Myr, 3)
        self.assertAlmostEqual(instance.particles.time_step, [550.1565, 58.2081, 18.8768] | units.Myr, 3)
        self.assertAlmostEqual(instance.model_time, 18.8768 | units.Myr, 3)

        print("evolve_model with end_time: take timesteps, until end_time is reached exactly")
        instance.evolve_model(100 | units.Myr)
        self.assertAlmostEqual(instance.particles.age, [100.0, 100.0, 100.0] | units.Myr, 3)
        self.assertAlmostEqual(instance.particles.time_step, [550.1565, 58.2081, 18.8768] | units.Myr, 3)
        self.assertAlmostEqual(instance.model_time, 100.0 | units.Myr, 3)

        print("evolve_model with keep_synchronous: use non-shared timestep, particle ages will typically diverge")
        instance.evolve_model(keep_synchronous=False)
        self.assertAlmostEqual(instance.particles.age, (100 | units.Myr) + ([550.1565, 58.2081, 18.8768] | units.Myr), 3)
        self.assertAlmostEqual(instance.particles.time_step, [550.1565, 58.2081, 18.8768] | units.Myr, 3)
        self.assertAlmostEqual(instance.model_time, 100.0 | units.Myr, 3)  # Unchanged!
        instance.stop()

    def test12(self):
        print("Testing adding and removing particles from stellar evolution code...")

        particles = Particles(3)
        particles.mass = 1.0 | units.MSun

        instance = SSE()
        instance.initialize_code()
        instance.commit_parameters()
        self.assertEqual(len(instance.particles), 0)  # before creation
        instance.particles.add_particles(particles[:-1])
        instance.commit_particles()
        instance.evolve_model(1.0 | units.Myr)
        self.assertEqual(len(instance.particles), 2)  # before remove
        self.assertAlmostEqual(instance.particles.age, 1.0 | units.Myr)

        instance.particles.remove_particle(particles[0])
        self.assertEqual(len(instance.particles), 1)
        instance.evolve_model(2.0 | units.Myr)
        self.assertAlmostEqual(instance.particles[0].age, 2.0 | units.Myr)

        instance.particles.add_particles(particles[::2])
        self.assertEqual(len(instance.particles), 3)  # it's back...
        self.assertAlmostEqual(instance.particles[0].age, 2.0 | units.Myr)
        self.assertAlmostEqual(instance.particles[1].age, 0.0 | units.Myr)
        self.assertAlmostEqual(instance.particles[2].age, 0.0 | units.Myr)  # ... and rejuvenated.

        instance.evolve_model(3.0 | units.Myr)  # The young stars keep their age offset from the old star
        self.assertAlmostEqual(instance.particles.age, [3.0, 1.0, 1.0] | units.Myr)
        instance.evolve_model(4.0 | units.Myr)
        self.assertAlmostEqual(instance.particles.age, [4.0, 2.0, 2.0] | units.Myr)
        instance.stop()

    def test13(self):
        print("Testing SSE states")
        stars = Particles(1)
        stars.mass = 1.0 | units.MSun

        print("First do everything manually:", end=' ')
        instance = SSE()
        self.assertEqual(instance.get_name_of_current_state(), 'UNINITIALIZED')
        instance.initialize_code()
        self.assertEqual(instance.get_name_of_current_state(), 'INITIALIZED')
        instance.commit_parameters()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        instance.cleanup_code()
        self.assertEqual(instance.get_name_of_current_state(), 'END')
        instance.stop()
        print("ok")

        print("initialize_code(), commit_parameters(), "
            "and cleanup_code() should be called automatically:", end=' ')
        instance = SSE()
        self.assertEqual(instance.get_name_of_current_state(), 'UNINITIALIZED')
        instance.parameters.reimers_mass_loss_coefficient = 0.5
        self.assertEqual(instance.get_name_of_current_state(), 'INITIALIZED')
        instance.particles.add_particles(stars)
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        instance.stop()
        self.assertEqual(instance.get_name_of_current_state(), 'STOPPED')
        print("ok")

    def test14a(self):
        print("Testing basic operations: evolve_one_step and evolve_for (on particle)")
        stars = Particles(2)
        stars.mass = 1.0 | units.MSun
        instance = SSE()
        se_stars = instance.particles.add_particles(stars)
        self.assertAlmostEqual(se_stars.age, [0.0, 0.0] | units.yr)

        for i in range(3):
            se_stars[0].evolve_one_step()
        self.assertAlmostEqual(se_stars.age, [1650.46953688, 0.0] | units.Myr, 3)
        number_of_steps = 10
        step_size = se_stars[0].age / number_of_steps
        for i in range(1, number_of_steps + 1):
            se_stars[1].evolve_for(step_size)
            self.assertAlmostEqual(se_stars.age, [number_of_steps, i] * step_size)
        self.assertAlmostRelativeEqual(se_stars[0].age,         se_stars[1].age)
        self.assertAlmostRelativeEqual(se_stars[0].luminosity,  se_stars[1].luminosity, 3)
        self.assertAlmostRelativeEqual(se_stars[0].radius,      se_stars[1].radius, 3)
        self.assertAlmostRelativeEqual(se_stars[0].temperature, se_stars[1].temperature, 3)
        instance.stop()

    def test14b(self):
        print("Testing basic operations: evolve_one_step and evolve_for (on subset)")
        stars = Particles(2)
        stars.mass = 1.0 | units.MSun
        instance = SSE()
        se_stars = instance.particles.add_particles(stars)
        self.assertAlmostEqual(se_stars.age, [0.0, 0.0] | units.yr)

        for i in range(3):
            se_stars[:1].evolve_one_step()
        self.assertAlmostEqual(se_stars.age, [1650.46953688, 0.0] | units.Myr, 3)
        number_of_steps = 10
        step_size = se_stars[0].age / number_of_steps
        for i in range(1, number_of_steps + 1):
            se_stars[1:].evolve_for(step_size)
            self.assertAlmostEqual(se_stars.age, [number_of_steps, i] * step_size)
        self.assertAlmostRelativeEqual(se_stars[0].age,         se_stars[1].age)
        self.assertAlmostRelativeEqual(se_stars[0].luminosity,  se_stars[1].luminosity, 3)
        self.assertAlmostRelativeEqual(se_stars[0].radius,      se_stars[1].radius, 3)
        self.assertAlmostRelativeEqual(se_stars[0].temperature, se_stars[1].temperature, 3)
        instance.stop()

    def test15(self):
        print("test evolution of 1000 star sampled over flattish IMF")

        number_of_stars = 1000

        class notsorandom(object):
            def random(self, N):
                return numpy.array(range(N))/(N-1.)

            def random_sample(self, N):
                return numpy.array(range(N))/(N-1.)

        masses = new_salpeter_mass_distribution(
            number_of_stars,
            mass_min=0.1 | units.MSun,
            mass_max=100.0 | units.MSun,
            alpha=-1.01, random=notsorandom()
        )

        stars = Particles(mass=masses)

        instance = SSE()
        instance.particles.add_particles(stars)

        i = 0
        for p in instance.particles:
            p.evolve_for(0.1 | units.Myr)
            i += 1
        instance.stop()

    def test16(self):
        print("test evolution of 1000 star sampled over flattish IMF")

        number_of_stars = 1000

        class notsorandom(object):
            def random(self, N):
                return numpy.array(range(N))/(N-1.)

            def random_sample(self, N):
                return numpy.array(range(N))/(N-1.)

        masses = new_salpeter_mass_distribution(
            number_of_stars,
            mass_min=0.1 | units.MSun,
            mass_max=100.0 | units.MSun,
            alpha=-1.01, random=notsorandom()
        )

        stars = Particles(mass=masses)

        instance = SSE()
        instance.particles.add_particles(stars)

        i = 0
        for p in instance.particles:
            p.evolve_for(13.2 | units.Gyr)
            i += 1
        instance.stop()

    def test17(self):
        print("evolve_one_step and evolve_for after particle removal and addition")
        particles = Particles(10)
        particles.mass = range(1, 11) | units.MSun
        instance = SSE()
        instance.particles.add_particles(particles)
        self.assertAlmostEqual(instance.particles.age, 0.0 | units.yr)
        time_steps = numpy.linspace(0.1, 1.0, num=10) | units.Myr
        for i in range(10):
            instance.particles[i].evolve_for(time_steps[i])
        self.assertAlmostEqual(instance.particles.age, time_steps)

        instance.particles.remove_particles(particles[[1, 4, 8]])
        revived = instance.particles.add_particle(particles[4])
        revived.evolve_for(numpy.pi | units.Myr)
        for star in instance.particles:
            star.evolve_for(star.age)
        self.assertAlmostEqual(instance.particles.age[:-1], 2*time_steps[[0, 2, 3, 5, 6, 7, 9]])
        self.assertAlmostEqual(instance.particles.age[-1], 2*numpy.pi | units.Myr)

        instance.particles.remove_particles(particles[[2, 5, 6]])
        instance.particles.add_particles(particles[[8, 1]])
        self.assertEqual(len(instance.particles), 7)
        expected_ages = instance.particles.age + instance.particles.time_step
        for star in instance.particles:
            star.evolve_one_step()
        self.assertAlmostEqual(instance.particles.age, expected_ages)
        instance.stop()

    def test18(self):
        print("SSE validation")
        sse_src_path = os.path.join(os.path.dirname(sys.modules[SSE.__module__].__file__), 'src')
        if not os.path.exists(os.path.join(sse_src_path, "evolve.in")):
            self.skip("Not in a source release")
        instance = SSE()
        instance.particles.add_particle(Particle(mass=1.416 | units.MSun))
        instance.particles[0].evolve_for(7000.0 | units.Myr)
        evolved_star = instance.particles.copy()[0]
        # See issue #957
        # evolved_star.temperature = instance.particles[0].temperature
        instance.stop()

        testpath = get_path_to_results()
        shutil.copy(os.path.join(sse_src_path, "evolve.in"), os.path.join(testpath, "evolve.in"))

        call([os.path.join(sse_src_path, "sse")], cwd=testpath)

        with open(os.path.join(testpath, "evolve.dat"), "r") as sse_output:
            lines = sse_output.readlines()
            sse_final_result = lines[-2].split()

        self.assertAlmostEqual(evolved_star.age, float(sse_final_result[0]) | units.Myr, 3)
        self.assertAlmostEqual(evolved_star.stellar_type, float(sse_final_result[1]) | units.stellar_type, 3)
        self.assertAlmostEqual(evolved_star.initial_mass, float(sse_final_result[2]) | units.MSun, 3)
        self.assertAlmostEqual(evolved_star.mass, float(sse_final_result[3]) | units.MSun, 3)
        self.assertAlmostEqual(evolved_star.luminosity, 10**float(sse_final_result[4]) | units.LSun, 3)
        self.assertAlmostEqual(evolved_star.radius, 10**float(sse_final_result[5]) | units.RSun, 3)
        self.assertAlmostRelativeEqual(evolved_star.temperature, 10**float(sse_final_result[6]) | units.K, 2)
        self.assertAlmostEqual(evolved_star.core_mass, float(sse_final_result[7]) | units.MSun, 3)
        self.assertAlmostEqual(evolved_star.convective_envelope_mass, float(sse_final_result[8]) | units.MSun, 3)
        self.assertAlmostEqual(evolved_star.epoch, float(sse_final_result[9]) | units.Myr, 3)
        self.assertAlmostEqual(evolved_star.spin, float(sse_final_result[10]) | units.yr**-1, 3)

    def test19(self):
        print("SSE core_mass and CO_core_mass (high mass star)")
        instance = SSE()
        star = instance.particles.add_particle(Particle(mass=30 | units.MSun))
        instance.evolve_model(5.8 | units.Myr)
        print(star.mass, star.core_mass, star.CO_core_mass, star.stellar_type)
        self.assertEqual(str(star.stellar_type), "Main Sequence star")
        self.assertIsOfOrder(star.mass, 30 | units.MSun)
        self.assertEqual(star.core_mass, 0 | units.MSun)
        self.assertEqual(star.CO_core_mass, 0 | units.MSun)
        instance.evolve_model(6.0 | units.Myr)
        print(star.mass, star.core_mass, star.CO_core_mass, star.stellar_type)
        self.assertEqual(str(star.stellar_type), "Core Helium Burning")
        self.assertIsOfOrder(star.mass, 30 | units.MSun)
        self.assertIsOfOrder(star.core_mass, 10 | units.MSun)
        self.assertEqual(star.CO_core_mass, 0 | units.MSun)
        instance.evolve_model(6.5 | units.Myr)
        print(star.mass, star.core_mass, star.CO_core_mass, star.stellar_type)
        self.assertEqual(str(star.stellar_type), "Main Sequence Naked Helium star")
        self.assertIsOfOrder(star.mass, 10 | units.MSun)
        self.assertEqual(star.core_mass, star.mass)
        self.assertEqual(star.CO_core_mass, 0 | units.MSun)
        instance.evolve_model(6.65 | units.Myr)
        print(star.mass, star.core_mass, star.CO_core_mass, star.stellar_type)
        self.assertEqual(str(star.stellar_type), "Hertzsprung Gap Naked Helium star")
        self.assertIsOfOrder(star.mass, 10 | units.MSun)
        self.assertEqual(star.core_mass, star.mass)
        self.assertAlmostEqual(star.CO_core_mass, 7.12 | units.MSun, 2)
        instance.evolve_model(7.0 | units.Myr)
        print(star.mass, star.core_mass, star.CO_core_mass, star.stellar_type)
        self.assertEqual(str(star.stellar_type), "Black Hole")
        self.assertIsOfOrder(star.mass, 10 | units.MSun)
        self.assertEqual(star.core_mass, star.mass)
        self.assertEqual(star.CO_core_mass, star.mass)
        instance.stop()

    def test20(self):
        print("SSE core_mass and CO_core_mass (low mass stars)")
        instance = SSE()
        stars = instance.particles.add_particles(Particles(mass=[0.6, 1.0] | units.MSun))
        instance.evolve_model(100 | units.Gyr)
        self.assertEqual(str(stars[0].stellar_type), "Helium White Dwarf")
        self.assertAlmostEqual(stars[0].mass, 0.405 | units.MSun, 2)
        self.assertEqual(stars[0].core_mass, stars[0].mass)
        self.assertEqual(stars[0].CO_core_mass, 0 | units.MSun)
        self.assertEqual(str(stars[1].stellar_type), "Carbon/Oxygen White Dwarf")
        self.assertAlmostEqual(stars[1].mass, 0.520 | units.MSun, 2)
        self.assertEqual(stars[1].core_mass, stars[1].mass)
        self.assertEqual(stars[1].CO_core_mass, stars[1].mass)
        instance.stop()

    def test21(self):
        instance = SSE()
        stars = instance.particles.add_particles(Particles(mass=30 | units.MSun))
        mass_loss_wind = stars[0].mass_loss_wind
        self.assertAlmostRelativeEquals(mass_loss_wind, 1.703e-07 | units.MSun / units.yr, 3)
        instance.evolve_model(1 | units.Myr)
        dm = (1 | units.Myr) * mass_loss_wind
        self.assertAlmostRelativeEquals(stars[0].mass, (30 | units.MSun) - dm,  3)
        self.assertAlmostRelativeEquals(stars[0].mass_loss_wind, 2.053e-07 | units.MSun / units.yr, 3)

        instance.stop()

    def test22(self):
        instance = SSE()
        stars = instance.particles.add_particles(Particles(mass=[1.0, 10.0] | units.MSun))
        gyration_radius = stars.gyration_radius
        self.assertTrue(numpy.all(0.0 < gyration_radius))
        self.assertTrue(numpy.all(gyration_radius < 1.0))
        instance.evolve_model(12.4 | units.Gyr)
        self.assertTrue(stars[0].gyration_radius < gyration_radius[0])
        self.assertTrue(stars[1].gyration_radius > gyration_radius[1])
        instance.evolve_model(14 | units.Gyr)
        self.assertTrue(numpy.all(stars.gyration_radius > gyration_radius))
        instance.stop()

    def test23(self):
        instance = SSE()
        p = Particles(mass=[1.0, 10.0] | units.MSun, temperature=[10, 10] | units.K)
        stars = instance.particles.add_particles(p)
        channel = stars.new_channel_to(p)
        channel.copy_attributes(["mass", "temperature"])
        self.assertEqual(stars.temperature, p.temperature)
        instance.stop()



import sys
import os
import numpy.random

from amuse.test import amusetest
from amuse.units import units, nbody_system
from amuse.ext.boss_bodenheimer import bb79_cloud

numpy.random.seed(1234567)


class BossBodenheimerTests(amusetest.TestCase):
    def test1(self):
        numpy.random.seed(1234)
        mc = bb79_cloud(targetN=1000).result
        self.assertEqual(len(mc), 1000)
        ek = mc.kinetic_energy()
        ep = mc.potential_energy(G=nbody_system.G)
        eth = mc.thermal_energy()
        self.assertAlmostEqual(eth/ep, -0.25, 2)
        self.assertAlmostEqual(ek/ep, -0.2, 2)

    def test2(self):
        numpy.random.seed(1234)
        convert = nbody_system.nbody_to_si(1. | units.MSun, 3.2e16 | units.cm)
        mc = bb79_cloud(targetN=1000, convert_nbody=convert).result
        self.assertEqual(len(mc), 1000)
        ek = mc.kinetic_energy()
        ep = mc.potential_energy()
        eth = mc.thermal_energy()
        self.assertAlmostEqual(eth/ep, -0.25, 2)
        self.assertAlmostEqual(ek/ep, -0.2, 2)





import sys
import numpy

from optparse import OptionParser

try:
    from matplotlib import pyplot
    HAS_MATPLOTLIB = True
except ImportError:
    HAS_MATPLOTLIB = False

def select(row, cols_spec):
    subspecs = cols_spec.split(',')
    subspecs = list(map(str.strip, subspecs))
    cols = []
    for subspec in subspecs:
        parts = subspec.split('-')
        if len(parts) == 1:
            cols.append(int(parts[0]))
        else:
            if len(parts[1]) == 0:
                end = len(row)
            else:
                end = int(parts[1])
                if end < 0:
                    end = len(row) + end
            cols.extend(list(range(int(parts[0]), end)))
    for index in cols:
        yield row[index]
        
def plot_speed_report(input_filename = None, output_filename = None, cols = '0-'):
    with open(input_filename, 'r') as stream:
        lines = stream.readlines()
    header = None
    x = []
    data = []
    for line in lines:
        if line.startswith('#'):
            header_for_next_line = line[1:].split(',')
            header_for_next_line = list(select(header_for_next_line[2:], cols))
            if not header is None:
                if not header == header_for_next_line:
                    raise Exception("data does not have same header")
            header = header_for_next_line
        else:
            parts = list(map(str.strip, line.split(',')))
            if parts[0]== '':
                continue
            x.append(int(parts[0]))
            numbers = [float(x) for x in parts[2:]]
            data.append(list(select(numbers, cols)))
    x = numpy.asarray(x)
    data = numpy.asarray(data)
    print(data.shape)
    
    figure = pyplot.figure(figsize=(9, 4))
    subplot = pyplot.subplot(1,2,1)
    
    handles = subplot.plot(x,data)
    subplot.plot(x,1e-5 * x)

    subplot.legend(
        handles, 
        header,
        loc='center left', 
        bbox_to_anchor=(1.05, 0.5),
        ncol=1, 
        fancybox=False, 
        shadow=False)
    pyplot.loglog()
    if output_filename is None:
        pyplot.show()
    else:
        pyplot.savefig(output_filename)
                       
def new_option_parser():
    result = OptionParser()
    result.add_option(
        "-o", 
        default = None,
        dest="output_filename",
        help="save figure to output, by default it will display it",
        type="string"
    )
    result.add_option(
        "-i", 
        default = 'report.csv',
        dest="input_filename",
        help="name of the file to load the data from",
        type="string"
    )
    result.add_option(
        "--cols", 
        default = '0-',
        dest="cols",
        help="columns to plot, can by 1,2,3 or 0-3 or 0-5, 6, 3",
        type="string"
    )
    
    return result
    
    
    
if __name__ == '__main__':
    options, arguments = new_option_parser().parse_args()
    plot_speed_report(**options.__dict__)
